/* sane - Scanner Access Now Easy.

   Copyright (C) 2000 Mustek.
   Maintained by Tom Wang <tom.wang@mustek.com.tw>

   Updates (C) 2001 by Henning Meier-Geinitz.

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB flatbed scanners.  */
  
#ifndef mustek_usb_high_h
#define mustek_usb_high_h
#include "mustek_usb_mid.h"

/* ---------------------------------- macros ------------------------------ */

#define A4CIS_MinExploseTime 2700
#define A4CIS_Infinite 0xefff
#define A4CIS_300AdjustLength 2560
#define A4CIS_600AdjustLength 5120
#define A4CIS_GrayChannel CH_GREEN
#define A4CIS_GrayKLine pGreenKStrip 
#define A4CIS_GrayKBase pGreenKOffset
#define A4CIS_GrayLine pGreen	
#define A4CIS_SkipsPerRow 12   /* This value must be Times of 6 */
#define A4CIS_BytesPerStrip (8*1024L)
#define A4CIS_KLines 16
#define A4CIS_KLoops 2
#define A4CIS_KFilter 3
#define A4CIS_JLines 60
#define A4CIS_KLevel 250
#define A4CIS_PowerDelayLines 3
#define A4CIS_HomeLines 80
#define A4CIS_MaxPowerDelay 245
#define A4CIS_BlackFactor 0
#define A4CIS_GreenFactor "1.0"
#define A4CIS_BlueFactor "1.0"
#define A4CIS_RedFactor "1.0"
#define A4CIS_GrayFactor "1.0"

#define I8O8RGB 0
#define I8O8MONO 1
#define I4O1MONO 2

#define CURMOTOR CAdaMotor1200

/* ---------------------------------- types ------------------------------- */


struct Mustek_Usb_Device;

typedef SANE_Status
(* usb_high_PPOWERDELAYFUNC) (ma1017 *, SANE_Byte);

typedef void 
(* mustek_usb_high_PPGAFUNC) (ma1017 *, int);

typedef SANE_Status
(* usb_high_scan_GETLINEFUNC) (struct Mustek_Usb_Device * dev, SANE_Byte *, 
			       SANE_Bool isOrderInvert);

typedef SANE_Status
(* usb_high_scan_BACKTRACKFUNC) (struct Mustek_Usb_Device * dev); 

typedef enum tagSCANNERSTATE 
{
  ST_OFFLINE=0,
  ST_READY=1,
  ST_BUSY=2
} 
SCANNERSTATE, *PSCANNERSTATE;

typedef enum tagSENSORSTATE 
{
  SS_UNKNOW=0,
  SS_ON=1,
  SS_OFF=2
} 
SENSORSTATE, *PSENSORSTATE;

typedef enum tagCOLORMODE
{
  CRGB48=0,
  CRGB42=1,
  CRGB36=2,
  CRGB30=3,
  CRGB24=4,
  CGRAY16=5,
  CGRAY14=6,
  CGRAY12=7,
  CGRAY10=8,
  CGRAY8=9,
  CTEXT=10,
  CRGB48ext=11,
  CRGB42ext=12,
  CRGB36ext=13,
  CRGB30ext=14,
  CRGB24ext=15,
  CGRAY16ext=16,
  CGRAY14ext=17,
  CGRAY12ext=18,
  CGRAY10ext=19,
  CGRAY8ext=20,
  CTEXText=21
} 
COLORMODE, *PCOLORMODE;

typedef struct tagSCANNERINFORMATION
{
  SANE_Byte bMajor;
  SANE_Byte bMinor;
  SANE_Word wADCBits;
  SANE_Word wInputGammaBits;
  SANE_Word wOutputGammaBits;
  char sSensorName[16];
  char sAsicName[16];
}
SCANNERINFORMATION, *PSCANNERINFORMATION;

typedef enum tagIOTAG 
{
  IO_NONE=0x00,
  IO_PARALLEL=0x01,
  IO_USB=0x02,
  IO_SCSI=0x04,
  IO_SERIAL=0x08
} 
IOTAG, *PIOTAG;

typedef struct tagIOTYPE 
{
  IOTAG ioTag;
} 
IOTYPE, *PIOTYPE;

typedef enum tagPARAPORTMODE
{
  PPNone=0,
  PPNibble=1,
  PPUniDirection=2,
  PPBiDirection=3,
  PPEPP=4,
  PPECP=5
} 
PARAPORTMODE, *PPARAPORTMODE;

typedef struct tagPARAPORTINFORMATION
{
  IOTAG ioTag;
  PARAPORTMODE ppmParaPort;
  SANE_Word wLptNum;
} 
PARAPORTINFORMATION, *PPARAPORTINFORMATION;

typedef enum tagUSBENDPOINTTYPE 
{
  UEP_NONE=0,
  UEP_CONTROL=1,
  UEP_BULKIN=2,
  UEP_BULKOUT=3,
  UEP_INTERRUPTIN=4,
  UEP_INTERRUPTOUT=5,
  UEP_ISOCHRONOUSIN=6,
  UEP_ISOCHRONOUSOUT=7
} 
USBENDPOINTTYPE, *PUSBENDPOINTTYPE;

typedef struct tagUSBINFORMATION 
{
  IOTAG ioTag;
  SANE_Word wNumOfEndPoints;
  USBENDPOINTTYPE uepUsbEndPoint[16];
} 
USBINFORMATION, *PUSBINFORMATION;

typedef union tagIOINFORMATION 
{
  IOTYPE ioType;
  PARAPORTINFORMATION ioParallel;
  USBINFORMATION ioUSB;
} 
IOINFORMATION, *PIOINFORMATION;

typedef struct tagTARGETIMAGE
{
  SANE_Bool isOptimalSpeed;
  COLORMODE cmColorMode;
  SANE_Word wDpi;
  SANE_Word wX;
  SANE_Word wY;
  SANE_Word wWidth;
  SANE_Word wHeight;
} 
TARGETIMAGE, *PTARGETIMAGE;

typedef struct tagSUGGESTSETTING 
{
  COLORMODE cmScanMode;
  SANE_Word wXDpi;
  SANE_Word wYDpi;
  SANE_Word wX;
  SANE_Word wY;
  SANE_Word wWidth;
  SANE_Word wHeight;
  SANE_Word dwBytesPerRow;
} 
SUGGESTSETTING, *PSUGGESTSETTING;


/* end IScanner.h Inserts */

typedef enum tagDIRECTION 
{
  DR_FORWARD=0,
  DR_BACKWARD=1
} 
DIRECTION;

typedef enum tagSIGNALSTATE
{
  SG_UNKNOW=0,
  SG_BRIGHTER=1,
  SG_DARKER=2,
  SG_EQUAL=3
}
SIGNALSTATE;

typedef struct tagCCalibratorMD12
{
  /* Calibration Data */
  SANE_Bool isPrepared;
  SANE_Word* pKWhite;
  SANE_Word* pKDark;
  /* Working Buffer */
  double* pWhiteLine;
  double* pDarkLine;
  /* int** pWhiteBuffer; */
  int *pWhiteBuffer;
  /* Necessary Parameters */
  SANE_Word wKWhiteLevel;
  SANE_Word wKDarkLevel;
  SANE_Word wMajorAver;
  SANE_Word wMinorAver;
  SANE_Word wFilter;
  SANE_Word wWhiteNeeded;
  SANE_Word wDarkNeeded;
  SANE_Word wMaxWidth;
  SANE_Word wWidth;
  SANE_Word wThreshold;
  SANE_Word* pGammaTable;
  SANE_Byte nCalibratorType;
}
CalMD12;

enum Mustek_Usb_Modes
{
  MUSTEK_USB_MODE_LINEART=0,
  MUSTEK_USB_MODE_GRAY,
  MUSTEK_USB_MODE_COLOR
};
  
enum Mustek_Usb_Option
{
  OPT_NUM_OPTS = 0,
  
  OPT_MODE_GROUP,
  OPT_MODE,
  OPT_RESOLUTION,
  OPT_PREVIEW,
  
  OPT_GEOMETRY_GROUP,
  OPT_TL_X,			/* top-left x */
  OPT_TL_Y,			/* top-left y */
  OPT_BR_X,			/* bottom-right x */
  OPT_BR_Y,			/* bottom-right y */
  
  OPT_ENHANCEMENT_GROUP,
  OPT_BRIGHTNESS,
  OPT_CONTRAST,
  OPT_CUSTOM_GAMMA,
  OPT_GAMMA_VECTOR_R,
  OPT_GAMMA_VECTOR_G,
  OPT_GAMMA_VECTOR_B,
  /* must come last: */
  NUM_OPTIONS
};

typedef union
{
  SANE_Word w;
  SANE_Word *wa;		/* word array */
  SANE_String s;
}
Option_Value;

typedef struct Mustek_Usb_Device
{
  struct Mustek_Usb_Device *next;
  SANE_String name;
  SANE_Device sane;
  SANE_Range dpi_range;
  SANE_Range x_range;
  SANE_Range y_range;
  /* maxwidth & max height in 300 dpi*/    
  SANE_Int MaxWidth;
  SANE_Int MaxHeight;

  ma1017 * chip; /* registers of the scanner controller chip */

  SANE_Byte *scan_buffer;
  SANE_Byte *scan_buffer_start;
  size_t scan_buffer_len;

  SENSOR Sensor;
  SANE_Bool bAlreadyDetectCIS;
  SUGGESTSETTING Suggest;
  SANE_Word mapping_table[256];
  SANE_Bool Prepared;

  SANE_Word INI_MinExploseTime;
  SANE_Word INI_300SkipsPerRow;
  SANE_Word INI_600SkipsPerRow;
  SANE_Word INI_JLines;
  SANE_Word INI_KLines;
  SANE_Word INI_KFilter;
  SANE_Int INI_KLoops;
  SANE_Word INI_PixelRateLines;
  SANE_Word INI_PixelRateFilts;
  SANE_Word INI_PowerDelayLines;
  SANE_Word INI_HomeLines;
  SANE_Word INI_DarkLines;
  SANE_Word INI_KLevel;
  SANE_Byte INI_MaxPowerDelay;
  SANE_Byte INI_MinPowerDelay;
  SANE_Byte INI_AdjustWay;
  double INI_GreenBlackFactor;
  double INI_BlueBlackFactor;
  double INI_RedBlackFactor;
  double INI_GrayBlackFactor; 
  double INI_GreenFactor;
  double INI_BlueFactor;
  double INI_RedFactor;
  double INI_GrayFactor;

  int INI_RedRgb600PGA;
  int INI_GreenRgb600PGA;
  int INI_BlueRgb600PGA;
  int INI_Mono600PGA;
  int INI_RedRgb300PGA;
  int INI_GreenRgb300PGA;
  int INI_BlueRgb300PGA;
  int INI_Mono300PGA;
  SANE_Word INI_ExploseTime;
  SANE_Byte INI_RedRgb600PowerDelay;
  SANE_Byte INI_GreenRgb600PowerDelay;
  SANE_Byte INI_BlueRgb600PowerDelay;
  SANE_Byte INI_RedMono600PowerDelay;
  SANE_Byte INI_GreenMono600PowerDelay;
  SANE_Byte INI_BlueMono600PowerDelay;
  SANE_Byte INI_RedRgb300PowerDelay;
  SANE_Byte INI_GreenRgb300PowerDelay;
  SANE_Byte INI_BlueRgb300PowerDelay;
  SANE_Byte INI_RedMono300PowerDelay;
  SANE_Byte INI_GreenMono300PowerDelay;
  SANE_Byte INI_BlueMono300PowerDelay;
  SANE_Byte INI_Threshold;
  
  SANE_Byte INI_TopRef;
  SANE_Byte INI_FrontEnd;
  SANE_Byte INI_RedOffset;
  SANE_Byte INI_GreenOffset;
  SANE_Byte INI_BlueOffset;
  
  int INI_Rgb36BitBackTrack;
  int INI_Rgb24BitBackTrack;
  int INI_Mono8BitBackTrack;
  int INI_Mono4BitBackTrack;
  int INI_Mono1BitBackTrack;
  
  SANE_Bool isOpened;
  SANE_Bool isPrepared;
  SANE_Word ExploseTime;
  SANE_Word Width;
  SANE_Word XDpi;
  SANE_Word YDpi;
  COLORMODE ScanMode;
  SANE_Word BytesPerRow;
  SANE_Word Dummy;
  SANE_Word BytesPerStrip;
  SANE_Byte *pImageBuffer;
  SANE_Byte *pRed;
  SANE_Byte *pGreen;
  SANE_Byte *pBlue;
  usb_high_scan_GETLINEFUNC pfGetLine;
  usb_high_scan_BACKTRACKFUNC pfBackTrack;
  SANE_Bool isAdjustedRgb600PowerDelay;
  SANE_Bool isAdjustedMono600PowerDelay;
  SANE_Bool isAdjustedRgb300PowerDelay;
  SANE_Bool isAdjustedMono300PowerDelay;
  SANE_Bool isEvaluePixelRate;
  int RedRgb600PGA;
  int GreenRgb600PGA;
  int BlueRgb600PGA;
  int Mono600PGA;
  SANE_Byte RedRgb600PowerDelay;
  SANE_Byte GreenRgb600PowerDelay;
  SANE_Byte BlueRgb600PowerDelay;
  SANE_Byte RedMono600PowerDelay;
  SANE_Byte GreenMono600PowerDelay;
  SANE_Byte BlueMono600PowerDelay;
  int RedRgb300PGA;
  int GreenRgb300PGA;
  int BlueRgb300PGA;
  int Mono300PGA;
  SANE_Byte RedRgb300PowerDelay;
  SANE_Byte GreenRgb300PowerDelay;
  SANE_Byte BlueRgb300PowerDelay;
  SANE_Byte RedMono300PowerDelay;
  SANE_Byte GreenMono300PowerDelay;
  SANE_Byte BlueMono300PowerDelay;
  SANE_Word PixelRate;
  SANE_Byte Threshold;
  SANE_Word * pGammaTable;
  SANE_Word SkipsPerRow;
  
  CalMD12 *pRedCalibrator;
  CalMD12 *pGreenCalibrator;
  CalMD12 *pBlueCalibrator;
  CalMD12 *pMonoCalibrator;

  SANE_Char DeviceName[256];

}
Mustek_Usb_Device;

typedef struct Mustek_Usb_Scanner
{
  /* all the state needed to define a scan request: */
  struct Mustek_Usb_Scanner *next;
  
  SANE_Option_Descriptor opt[NUM_OPTIONS];
  Option_Value val[NUM_OPTIONS];
  
  SANE_Int nChannels;
  
  double ulx;
  double uly;
  double wx;
  double wy;
  SANE_Int ULx;
  SANE_Int ULy;
  SANE_Int Width;
  SANE_Int Height;
  SANE_Bool scanning;
  SANE_Bool aborted_by_user;
  SANE_Parameters params;
  SANE_Word read_rows;

  /* scanner dependent/low-level state: */
  Mustek_Usb_Device *hw;
}
Mustek_Usb_Scanner;


/* ------------------------- function declarations ------------------------ */


static SANE_Status
usb_high_cal_MD12_init (CalMD12 * cal, SANE_Byte type, SANE_Word wTargetWhite,
			SANE_Word wTargetDark);

static SANE_Status
usb_high_cal_MD12_exit (CalMD12 * cal);

static SANE_Status
usb_high_cal_MD12_EmbedGamma (CalMD12 * cal, SANE_Word * pGammaTable);

static SANE_Status
usb_high_cal_MD12_Prepare (CalMD12 * cal, SANE_Word wMaxWidth);

static SANE_Status
usb_high_cal_MD12_Setup (CalMD12 * cal, SANE_Word wMojorAver,
			 SANE_Word wMinorAver, SANE_Word wFilter,
			 SANE_Word wWidth, SANE_Word* wWhiteNeeded,
			 SANE_Word* wDarkNeeded);

static SANE_Status
usb_high_cal_MD12_EvalueWhite (CalMD12 * cal, double dFactor);

static SANE_Status
usb_high_cal_MD12_EvalueDark (CalMD12 * cal, double dFactor);

static SANE_Status 
usb_high_cal_MD12_EvalueCalibrator (CalMD12 * cal);

static SANE_Status
usb_high_cal_MD12_FillInWhite (CalMD12 * cal, SANE_Word wMajor,
			       SANE_Word Minor,	void * pWhitePattern);

static SANE_Status
usb_high_cal_MD12_FillInDark (CalMD12 * cal, SANE_Word wMajor,
			      SANE_Word Minor, void * pDarkPattern);

static SANE_Status
usb_high_cal_MD12_Calibrate (CalMD12 * cal, void * pSrc, void * pTarget);

/* original virtual functions defined here */

static SANE_Status
usb_high_cal_MD12i8o8_FillInWhite (CalMD12 * cal, SANE_Word wMajor,
				   SANE_Word Minor, void * pWhitePattern);

static SANE_Status
usb_high_cal_MD12i8o8_FillInDark (CalMD12 * cal, SANE_Word wMajor, 
				  SANE_Word Minor, void * pDarkPattern);

static SANE_Status
usb_high_cal_MD12i8o8Mono_Calibrate (CalMD12 * cal, void * pSrc, 
				     void * pTarget);

static SANE_Status
usb_high_cal_MD12i8o8Rgb_Calibrate (CalMD12 * cal, void * pSrc,
				    void * pTarget);

static SANE_Status
usb_high_cal_MD12i4o1_FillInWhite (CalMD12 * cal,  SANE_Word wMajor,
				   SANE_Word Minor, void * pWhitePattern);

static SANE_Status
usb_high_cal_MD12i4o1_FillInDark (CalMD12 * cal, SANE_Word wMajor,
				  SANE_Word Minor, void * pDarkPattern);

static SANE_Status
usb_high_cal_MD12i4o1_Calibrate (CalMD12 * cal,void * pSrc,
				 void * pTarget);

static SANE_Status 
usb_high_scan_init (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_exit (Mustek_Usb_Device * dev);

static SANE_Status 
usb_high_scan_Prepare (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_Clearup (Mustek_Usb_Device * dev);

static SANE_Status 
usb_high_scan_TurnPower (Mustek_Usb_Device * dev, SANE_Bool isOn);

static SANE_Status 
usb_high_scan_BackHome (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_SetThreshold (Mustek_Usb_Device * dev, SANE_Byte threshold);

static SANE_Status
usb_high_scan_EmbedGamma (Mustek_Usb_Device * dev, SANE_Word * pGammaTable);

static SANE_Status 
usb_high_scan_Reset (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_ScanSuggest (Mustek_Usb_Device * dev, PTARGETIMAGE pTarget,
			   PSUGGESTSETTING pSuggest);
static SANE_Status 
usb_high_scan_detect_sensor (Mustek_Usb_Device *dev);

static SANE_Status 
usb_high_scan_SetupScan (Mustek_Usb_Device * dev, COLORMODE ColorMode,
			 SANE_Word XDpi, SANE_Word YDpi, SANE_Bool isInvert,
			 SANE_Word X, SANE_Word Y, SANE_Word Width);

static SANE_Status
usb_high_scan_GetRows (Mustek_Usb_Device * dev, SANE_Byte * lpBlock,
		       SANE_Word Rows, SANE_Bool isOrderInvert);

static SANE_Status 
usb_high_scan_StopScan (Mustek_Usb_Device * dev);	

static SANE_Status
usb_high_scan_StepForward (Mustek_Usb_Device * dev, int StepCount);

static SANE_Status
usb_high_scan_SafeForward (Mustek_Usb_Device * dev, int StepCount);

static SANE_Status
usb_high_scan_LoadPrivateProfile (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_InitASIC (Mustek_Usb_Device * dev, SENSOR Sensor);

static SANE_Status 
usb_high_scan_WaitCarriageHome (Mustek_Usb_Device * dev);

static SANE_Status 
usb_high_scan_HardwareCalibration (Mustek_Usb_Device * dev);

static SANE_Status 
usb_high_scan_LineCalibration (Mustek_Usb_Device * dev);

static SANE_Status 
usb_high_scan_PrepareScan (Mustek_Usb_Device * dev);

static SANE_Word 
usb_high_scan_CaculateMaxRgb600Explose (Mustek_Usb_Device * dev, 
					SANE_Byte*  IdealRedPD,
					SANE_Byte* IdealGreenPD,
					SANE_Byte* IdealBluePD);

static SANE_Word 
usb_high_scan_CaculateMaxMono600Explose (Mustek_Usb_Device * dev,
					 SANE_Byte* IdealRedPD,
					 SANE_Byte* IdealGreenPD,
					 SANE_Byte* IdealBluePD);

static SANE_Status
usb_high_scan_PrepareRgbSignal600Dpi (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_PrepareMonoSignal600Dpi (Mustek_Usb_Device * dev);

static SANE_Word 
usb_high_scan_CaculateMaxRgb300Explose (Mustek_Usb_Device * dev,
					SANE_Byte* IdealRedPD,
					SANE_Byte* IdealGreenPD,
					SANE_Byte* IdealBluePD);

static SANE_Word 
usb_high_scan_CaculateMaxMono300Explose (Mustek_Usb_Device * dev,
					 SANE_Byte* IdealRedPD,
					 SANE_Byte* IdealGreenPD,
					 SANE_Byte* IdealBluePD);

static SANE_Status
usb_high_scan_PrepareRgbSignal300Dpi (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_PrepareMonoSignal300Dpi (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_EvalueMaxLevel (Mustek_Usb_Device * dev, SANE_Word SampleLines,
			      int SampleLength, SANE_Byte *ret_max_level);

static SANE_Status 
usb_high_scan_BSSCPowerDelay (Mustek_Usb_Device * dev,
			      usb_high_PPOWERDELAYFUNC pSetPowerDelay,
			      SIGNALSTATE* ssCurrent, SANE_Byte* Target,
			      SANE_Byte Max, SANE_Byte Min,
			      SANE_Byte Threshold, int Length);

static SANE_Status
usb_high_scan_AdjustRgb600PowerDelay (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_AdjustMono600PowerDelay (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_AdjustRgb300PowerDelay (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_AdjustMono300PowerDelay (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_EvaluePixelRate (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_CalibrationRgb24Bit (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_CalibrationMono8Bit (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_PrepareRgb24Bit (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_PrepareMono8Bit (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_GetRgb24BitLine (Mustek_Usb_Device * dev,
			       SANE_Byte * lpLine,
			       SANE_Bool isOrderInvert);

static SANE_Status
usb_high_scan_GetMono8BitLine (Mustek_Usb_Device * dev,
			       SANE_Byte * lpLine,
			       SANE_Bool isOrderInvert);

static SANE_Status
usb_high_scan_BackTrackRgb24Bit (Mustek_Usb_Device * dev);

static SANE_Status
usb_high_scan_BackTrackMono8Bit (Mustek_Usb_Device * dev);

#endif /* mustek_usb_high_h */
