/* sane - Scanner Access Now Easy.

   Copyright (C) 2000 Mustek.
   Maintained by Tom Wang <tom.wang@mustek.com.tw>

   Updates (C) 2001 by Henning Meier-Geinitz.

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB flatbed scanners.  */

#include <unistd.h>

#include "../include/sane/sane.h"
#include "../include/sane/sanei_usb.h"
#include "mustek_usb_low.h"


SANE_Status
usb_low_init (ma1017 ** chip_address)
{
  SANE_Int i;
  ma1017 * chip;

  DBG (7, "usb_low_init: start\n");
  if (!chip_address)
    return SANE_STATUS_INVAL;

  chip = (ma1017 *) malloc (sizeof (ma1017));

  if (!chip)
    {
      DBG (3, "usb_low_init: couldn't malloc %d bytes for chip\n",
	   sizeof (ma1017));
      *chip_address = 0;
      return SANE_STATUS_NO_MEM;
    }
  *chip_address = chip;

  /* io */
  chip->isRowing = SANE_FALSE;
  chip->isOpened = SANE_FALSE;
  chip->fd = -1;

  /* Construction/Destruction */
  chip->isOpened = SANE_FALSE;
  chip->isRowing = SANE_FALSE;
  
  /* A2 */
  chip->bcmAppend = 0x00;
  chip->bcmTestSRAM = 0x00;
  chip->bcmFixPattern = 0x00;
  /* A4 */
  chip->bcmSelect = 0x00;
  chip->bcmFrontEnd = 0x00;
  /* A6 */
  chip->bcmRGBSelPin = 0x02;
  chip->bcmASICIoPins = 0x9c;
  /* A7 */
  chip->bcmTiming = 0xe8;
  chip->bcmSRAMBank = 0x02;
  /* A8 */
  chip->bcmDummyMSB = 0x00;
  chip->bcmCCDWidthMSB = 0x00;
  chip->bCMTTableLength = 0x00;
  /* A9 */
  chip->bCMTSecondPos = 0x00;
  /* A10 + A8ID5 */
  chip->wCCDWidth = 0x0c80;
  /* A11 + A8ID6 */
  chip->wDummy = 0x0020;
  /* A12 + A13 */
  chip->wByteWidth = 0x09f6;
  /* A14 + A30W */
  chip->wLoopCount = 0x0db5;
  /* A15 */
  chip->bcmMotorEnable = 0x00;
  chip->bcmMotorMovement = 0x60;
  chip->bcmMotorDirection = 0x10;
  chip->bcmMotorSignal = 0x00;
  chip->bcmMotorHome = 0x00;
  /* A16 */
  chip->bcmPixelDeepth = 0x00;
  chip->bcmImageInvert = 0x00;
  chip->bcmOptical600 = 0x00;
  chip->bcmSampleWay = 0x06;
  /* A17 + A18 + A19 */
  chip->bRedRef = 0xff;
  chip->bGreenRef = 0xff;
  chip->bBlueRef = 0xff;
  /* A20 + A21 + A22 */
  chip->bRedPD = 0x00;
  chip->bGreenPD = 0x00;
  chip->bBluePD = 0x00;
  /* A23 */
  chip->bcmA23 = 0x80;
  /* A24 */
  chip->bcmFy1Delay = 0x00;
  chip->bcmSpecialAD = 0x00;
  /* A27 */
  chip->bcmSCLK = 0x00;
  chip->bcmSEN = 0x00;
  chip->bSerialLength = 0x10;
  
  /* Use for Rowing */
  chip->pGetRow = NULL;
  
  chip->dwCMTTableLength = 0x00000000;
  chip->dwCMTSecondPos = 0x00000000;
  chip->wRowSize = 0x00;
  chip->wSoftResample = 0x01;
  chip->dwTotalLine = 0x00;
  chip->dwLeftLine = 0x00;
  for (i = 0; i < 32; i++)
    chip->isTransferTable[i] = SANE_FALSE;
  chip->Sensor = SR_CANON600;
  
  DBG (7, "usb_low_init: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_exit (ma1017 * chip)
{
  DBG (7, "usb_low_exit: chip = %p\n", chip);
  if (chip)
    {
      if (chip->fd >= 0)
	{
	  DBG (7, "usb_low_exit: closing fd %d\n", chip->fd);
	  sanei_usb_close (chip->fd);
	  chip->fd = -1;
	}
      DBG (7, "usb_low_exit: freeing chip\n");
      free (chip);
    }
  return SANE_STATUS_GOOD;
  DBG (7, "usb_low_exit: exit\n");
}

/* A0 ~ A1 */
SANE_Status
usb_low_SetCMTTable (ma1017 * chip, int iIndex, CHANNEL chChannel,
		     SANE_Bool isMoveMotor, SANE_Bool isTransfer)
{
  SANE_Byte bPattern = ((SANE_Byte) iIndex) << 4;
  SANE_Byte bRegNo = 0;
  CONTROLWORD CW_SetCMTTable;
  SANE_Status status;

  DBG (7, "usb_low_SetCMTTable: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetCMTTable: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetCMTTable: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  if ((unsigned int) iIndex > 31)
    {
      DBG (7, "usb_low_SetCMTTable: CMT index (%d) exceed 31", iIndex);
      return SANE_STATUS_INVAL;
    }

  switch (chChannel)
    {
    case CH_RED:
      bPattern |= 0x04;
      break;
    case CH_GREEN:
      bPattern |= 0x08;
      break;
    case CH_BLUE:
      bPattern |= 0x0c;
      break;
    default:
      break;
    }
  if (isMoveMotor)
    bPattern |= 0x02;
  if (isTransfer)
    bPattern |= 0x01;
  if (iIndex > 15)
    bRegNo++;
  CW_SetCMTTable.bByte = bPattern;
  CW_SetCMTTable.bRegNo = bRegNo;

  RIE(usb_low_write_reg (chip, CW_SetCMTTable));

  chip->isTransferTable[iIndex] = isTransfer;

  DBG (7, "usb_low_SetCMTTable: exit\n");

  return SANE_STATUS_GOOD;
}

/* A2 */
SANE_Status
usb_low_GetA2 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA2: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA2: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA2: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  RIE(usb_low_read_reg (chip, 2, &bPattern));

  chip->bcmAppend = bPattern & 0x10;
  chip->bcmTestSRAM = bPattern & 0x20;
  chip->bcmFixPattern = bPattern & 0x80;
  if (value)
    *value = bPattern;
  DBG (7, "usb_low_GetA2: exit, value =%d\n", bPattern);
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_StartCMTTable (ma1017 * chip)
{
  SANE_Byte bPattern;
  CONTROLWORD CW_StartCMTTable;
  size_t n;
  SANE_Status status;

  DBG (7, "usb_low_StartCMTTable: start\n");
  bPattern = 0x02 | chip->bcmAppend | chip->bcmTestSRAM | chip->bcmFixPattern;
  CW_StartCMTTable.bByte = bPattern;
  CW_StartCMTTable.bRegNo = 2;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_StartCMTTable: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (7, "usb_low_StartCMTTable: Already Rowing\n");
      return SANE_STATUS_INVAL;
    }

  CW_StartCMTTable.bRegNo |= 0x60;
  n = sizeof (CW_StartCMTTable);
  status = sanei_usb_write_bulk (chip->fd, (SANE_Byte *) &CW_StartCMTTable,
				 &n);
  if (status != SANE_STATUS_GOOD || n != sizeof (CW_StartCMTTable))
    {
      DBG (7, "usb_low_StartCMTTable: couldn't write: %s\n",
	   sane_strstatus (status));
      return SANE_STATUS_IO_ERROR;
    }
  chip->isRowing = SANE_TRUE;
  DBG (7, "usb_low_StartCMTTable: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_StopCMTTable (ma1017 * chip)
{
  SANE_Byte bPattern;
  CONTROLWORD CW_StopCMTTable;
  SANE_Byte ReadByte;
  size_t n;
  SANE_Status status;

  DBG (7, "usb_low_StopCMTTable: start\n");

  bPattern = 0x01 | chip->bcmAppend | chip->bcmTestSRAM | chip->bcmFixPattern;
  CW_StopCMTTable.bByte = bPattern;
  CW_StopCMTTable.bRegNo = 2;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_StopCMTTable: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (!chip->isRowing)
    {
      DBG (7, "usb_low_StopCMTTable: Not Rowing yet\n");
      return SANE_STATUS_INVAL;
    }

  CW_StopCMTTable.bRegNo |= 0x80;

  n = sizeof (CW_StopCMTTable);
  status = sanei_usb_write_bulk (chip->fd, (SANE_Byte *) &CW_StopCMTTable, &n);
  if (status != SANE_STATUS_GOOD || n != sizeof (CW_StopCMTTable))
    {
      DBG (1, "usb_low_StopCMTTable: couldn't write: %s\n",
	   sane_strstatus (status));
      return SANE_STATUS_IO_ERROR;
    }

  n = 1;
  status = sanei_usb_read_bulk (chip->fd, (SANE_Byte *) &ReadByte, &n);
  if (status != SANE_STATUS_GOOD || n != 1)
    {
      DBG (1, "usb_low_StopCMTTable: couldn't read: %s\n",
	   sane_strstatus (status));
      return SANE_STATUS_IO_ERROR;
    }

  chip->isRowing = SANE_FALSE;

  DBG (7, "usb_low_StopCMTTable: exit\n");

  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetTestSRAMMode (ma1017 * chip, SANE_Bool isTest)
{
  SANE_Byte bPattern;
  CONTROLWORD CW_SetTestSRAMMode;
  SANE_Status status;

  DBG (7, "usb_low_SetTestSRAMMode: start\n");

  bPattern = chip->bcmAppend | chip->bcmTestSRAM | chip->bcmFixPattern;
  CW_SetTestSRAMMode.bByte = bPattern;
  CW_SetTestSRAMMode.bRegNo = 2;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetTestSRAMMode: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetTestSRAMMode: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  if (isTest)
    chip->bcmTestSRAM = 0x20;
  else
    chip->bcmTestSRAM = 0x00;

  RIE(usb_low_write_reg (chip, CW_SetTestSRAMMode));

  DBG (7, "usb_low_SetTestSRAMMode: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetFixPattern (ma1017 * chip, SANE_Bool isFix)
{
  SANE_Byte bPattern;
  CONTROLWORD CW_SetFixPattern;
  SANE_Status status;

  DBG (7, "usb_low_SetFixPattern: start\n");

  bPattern = chip->bcmAppend | chip->bcmTestSRAM | chip->bcmFixPattern;
  CW_SetFixPattern.bByte = bPattern;
  CW_SetFixPattern.bRegNo = 2;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetFixPattern: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetFixPattern: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  if (isFix)
    chip->bcmFixPattern = 0x80;
  else
    chip->bcmFixPattern = 0x00;

  RIE(usb_low_write_reg (chip, CW_SetFixPattern));

  DBG (7, "usb_low_SetFixPattern: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_AdjustTiming (ma1017 * chip, SANE_Byte bAdjustPattern)
{
  CONTROLWORD CW_AdjustTiming;
  SANE_Status status;

  DBG (7, "usb_low_AdjustTiming: start\n");

  CW_AdjustTiming.bByte = bAdjustPattern;
  CW_AdjustTiming.bRegNo = 3;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_AdjustTiming: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_AdjustTiming: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_write_reg (chip, CW_AdjustTiming));

  DBG (7, "usb_low_AdjustTiming: exit\n");

  return SANE_STATUS_GOOD;
}

/* A4 */
SANE_Status
usb_low_GetA4 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Status status;
  SANE_Byte bPattern;

  DBG (7, "usb_low_GetA4: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA4: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA4: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 4, &bPattern));

  chip->bcmSelect = bPattern & 0xfe;
  chip->bcmFrontEnd = bPattern & 0x01;

  if (value)
    *value = bPattern;

  DBG (7, "usb_low_GetA4: exit, value=%d\n", bPattern);

  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SelectTiming (ma1017 * chip, SANE_Byte bSelectPattern)
{
  CONTROLWORD CW_SelectTiming;
  SANE_Status status;

  DBG (7, "usb_low_SelectTiming: start\n");

  CW_SelectTiming.bByte = bSelectPattern;
  CW_SelectTiming.bRegNo = 4;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SelectTiming: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SelectTiming: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmSelect = bSelectPattern & 0xfe;
  chip->bcmFrontEnd = bSelectPattern & 0x01;

  RIE(usb_low_write_reg (chip, CW_SelectTiming));
 
  DBG (7, "usb_low_SelectTiming: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_TurnFrontEndMode (ma1017 * chip, SANE_Bool isOn)
{
  CONTROLWORD CW_TurnFrontEndMode;
  SANE_Status status;

  DBG (7, "usb_low_TurnFrontEndMode: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_TurnFrontEndMode: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_TurnFrontEndMode: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  if (isOn)
    chip->bcmFrontEnd = 0x01;
  else
    chip->bcmFrontEnd = 0x00;

  CW_TurnFrontEndMode.bByte = chip->bcmSelect | chip->bcmFrontEnd;
  CW_TurnFrontEndMode.bRegNo = 4;

  RIE(usb_low_write_reg (chip, CW_TurnFrontEndMode));
  
  DBG (7, "usb_low_TurnFrontEndMode: exit\n");
  return SANE_STATUS_GOOD;
}

/* A6 */
SANE_Status 
usb_low_GetA6 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA6: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA6: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA6: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  RIE(usb_low_read_reg (chip, 6, &bPattern));

  chip->bcmASICIoPins = bPattern & 0xdc;
  chip->bcmRGBSelPin = bPattern & 0x03;

  if (value)
    *value = bPattern;

  DBG (7, "usb_low_GetA6: exit\n");

  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetASICIoPins (ma1017 * chip, SANE_Byte bIoPinPattern)
{
  CONTROLWORD CW_SetASICIoPins;
  SANE_Status status;

  DBG (7, "usb_low_SetASICIoPins: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetASICIoPins: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetASICIoPins: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmASICIoPins = bIoPinPattern & 0xdc;

  CW_SetASICIoPins.bByte = chip->bcmASICIoPins | chip->bcmRGBSelPin;
  CW_SetASICIoPins.bRegNo = 6;

  RIE(usb_low_write_reg (chip, CW_SetASICIoPins));
  
  DBG (7, "usb_low_SetASICIoPins: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetRGBSelPins (ma1017 * chip, SANE_Byte bRGBSelPinPattern)
{
  CONTROLWORD CW_SetRGBSelPin;
  SANE_Status status;
  
  DBG (7, "usb_low_SetRGBSelPin: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetRGBSelPin: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetRGBSelPin: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  chip->bcmRGBSelPin = bRGBSelPinPattern & 0x03;
  CW_SetRGBSelPin.bByte = chip->bcmASICIoPins | chip->bcmRGBSelPin;
  CW_SetRGBSelPin.bRegNo = 6;

  RIE(usb_low_write_reg (chip, CW_SetRGBSelPin));

  DBG (7, "usb_low_SetRGBSelPin: exit\n");
  return SANE_STATUS_GOOD; /* was false? */
}

/* A7 */
SANE_Status
usb_low_GetA7 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA7: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA7: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA7: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  RIE(usb_low_read_reg (chip, 7, &bPattern));

  if (value)
    *value = bPattern;

  chip->bcmTiming = bPattern & 0xfc;
  chip->bcmSRAMBank = bPattern & 0x03;

  DBG (7, "usb_low_GetA7: exit\n");

  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetTiming (ma1017 * chip, SANE_Byte bTimingPattern)
{
  CONTROLWORD CW_SetTiming;
  SANE_Status status;
  
  DBG (7, "usb_low_SetTiming: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetTiming: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetTiming: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmTiming = bTimingPattern & 0xfc;
  CW_SetTiming.bByte = chip->bcmTiming | chip->bcmSRAMBank;
  CW_SetTiming.bRegNo = 7;

  RIE(usb_low_write_reg (chip, CW_SetTiming));

  DBG (7, "usb_low_SetTiming: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetSRAMBank (ma1017 * chip, BANKSIZE bsBankSize)
{
  CONTROLWORD CW_SetSRAMBank;
  SANE_Status status;
  
  DBG (7, "usb_low_SetSRAMBank: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetSRAMBank: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetSRAMBank: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  switch (bsBankSize)
    {
    case BS_4K:
      chip->bcmSRAMBank = 0x00;
      break;
    case BS_8K:
      chip->bcmSRAMBank = 0x01;
      break;
    case BS_16K:
      chip->bcmSRAMBank = 0x02;
      break;
    default:
      DBG (3, "usb_low_SetSRAMBank: bsBankSize error\n");
      return SANE_STATUS_INVAL;
      break;
    }
  CW_SetSRAMBank.bByte = chip->bcmTiming | chip->bcmSRAMBank;
  CW_SetSRAMBank.bRegNo = 7;

  RIE(usb_low_write_reg (chip, CW_SetSRAMBank));
  
  DBG (7, "usb_low_SetSRAMBank: exit\n");
  return SANE_STATUS_GOOD;
}

/* A8 */
SANE_Status
usb_low_GetA8 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA8: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA8: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA8: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 8, &bPattern));

  chip->bcmDummyMSB = bPattern & 0x40;
  chip->bcmCCDWidthMSB = bPattern & 0x20;
  chip->bCMTTableLength = bPattern & 0x1f;
  chip->wCCDWidth =
    ((chip->wCCDWidth / 32) & 0x00ff) * 32 +
    ((chip->bcmCCDWidthMSB == 0) ? 0 : 0x0100 * 32);
  chip->wDummy =
    ((chip->wDummy / 32) & 0x00ff) * 32 +
    ((chip->bcmDummyMSB == 0) ? 0 : 0x0100 * 32);

  if (value)
    *value = bPattern;

  DBG (7, "usb_low_GetA8: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetCMTTableLength (ma1017 * chip, SANE_Byte bLength)
{
  CONTROLWORD CW_SetCMTTableLength;
  SANE_Status status;
  
  DBG (7, "usb_low_SetCMTTableLength: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetCMTTableLength: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetCMTTableLength: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  if (bLength > 32)
    {
      DBG (3, "usb_low_SetCMTTableLength: length %d exceeds 32\n",
	      (int) bLength);
      return SANE_STATUS_INVAL;
    }
  if (bLength == 0)
    {
      DBG (3, "usb_low_SetCMTTableLength: length is 0\n");
      return SANE_STATUS_INVAL;
    }

  chip->bCMTTableLength = bLength - 1;
  chip->dwCMTTableLength = (SANE_Word) bLength;
  CW_SetCMTTableLength.bByte =
    chip->bCMTTableLength | chip->bcmCCDWidthMSB |
    chip->bcmDummyMSB;
  CW_SetCMTTableLength.bRegNo = 8;

  RIE(usb_low_write_reg (chip, CW_SetCMTTableLength));
  
  DBG (7, "usb_low_SetCMTTableLength: exit\n");
  return SANE_STATUS_GOOD;
}

/* A9 */
SANE_Status
usb_low_GetA9 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA9: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA9: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA9: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  RIE(usb_low_read_reg (chip, 9, &bPattern));
 
  chip->bCMTSecondPos = bPattern & 0x1f;
  
  if (value)
    *value = bPattern;

  DBG (7, "usb_low_GetA9: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetCMTSecondPosition (ma1017 * chip, SANE_Byte bPosition)
{
  CONTROLWORD CW_SetCMTSecondPosition;
  SANE_Status status;
  
  DBG (7, "usb_low_SetCMTSecondPosition: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetCMTSecondPosition: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetCMTSecondPosition: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  if (bPosition > 31)
    {
      DBG (3, "usb_low_SetCMTTableLength: length: %d exceeds 31\n",
	      (int) bPosition);
      return SANE_STATUS_INVAL;
    }

  chip->bCMTSecondPos = bPosition;
  chip->dwCMTSecondPos = (SANE_Word) (bPosition);
  CW_SetCMTSecondPosition.bByte = chip->bCMTSecondPos;
  CW_SetCMTSecondPosition.bRegNo = 9;

  RIE(usb_low_write_reg (chip, CW_SetCMTSecondPosition));
  
  DBG (7, "usb_low_SetCMTSecondPosition: exit\n");

  return SANE_STATUS_GOOD;
}

/* A10 + A8ID5 */
SANE_Status
usb_low_GetA10 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA10: start\n");
      
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA10: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA10: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 10, &bPattern));

  chip->wCCDWidth =
    ((SANE_Word) (bPattern)) * 32 +
    ((chip->bcmCCDWidthMSB == 0) ? 0 : 0x0100 * 32);

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA10: exit\n");

  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetCCDWidth (ma1017 * chip, SANE_Word wCCDWidth)
{
  CONTROLWORD CW_SetCCDWidthMSB;
  CONTROLWORD CW_SetCCDWidth;
  SANE_Status status;
  
  DBG (7, "usb_low_SetCCDWidth: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetCCDWidth: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetCCDWidth: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  if (wCCDWidth / 32 > 0x01ff)
    {
      DBG (3, "usb_low_SetCCDWidth: width %d too high\n", (int) wCCDWidth);
      return SANE_STATUS_INVAL;
    }

  chip->wCCDWidth = wCCDWidth;
  wCCDWidth /= 32;
  if (HIBYTE (wCCDWidth) == 0x01)
    chip->bcmCCDWidthMSB = 0x20;
  else
    chip->bcmCCDWidthMSB = 0x00;
  CW_SetCCDWidthMSB.bByte =
    chip->bCMTTableLength | chip->bcmCCDWidthMSB |
    chip->bcmDummyMSB;
  CW_SetCCDWidthMSB.bRegNo = 8;
  CW_SetCCDWidth.bByte = LOBYTE (wCCDWidth);
  CW_SetCCDWidth.bRegNo = 10;

  RIE(usb_low_write_reg (chip, CW_SetCCDWidthMSB));
  
  RIE(usb_low_write_reg (chip, CW_SetCCDWidth));
  
  DBG (7, "usb_low_SetCCDWidth: exit\n");
  return SANE_STATUS_GOOD;
}

/* A11 + A8ID6 */
SANE_Status
usb_low_GetA11 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA11: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA11: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA11: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 11, &bPattern));

  chip->wDummy =
    ((SANE_Word) (bPattern)) * 32 +
    ((chip->bcmDummyMSB == 0) ? 0 : 0x0100 * 32);

  if (value)
    *value = bPattern;

  DBG (7, "usb_low_GetA11: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetDummy (ma1017 * chip, SANE_Word wDummy)
{
  CONTROLWORD CW_SetDummyMSB;
  CONTROLWORD CW_SetDummy;
  SANE_Status status;
  
  DBG (7, "usb_low_SetDummy: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetDummy: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetDummy: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  if (wDummy / 32 > 0x01ff)
    {
      DBG (7, "usb_low_SetDummy: width %d exceeded\n", (int) wDummy);
      return SANE_STATUS_INVAL;
    }

  chip->wDummy = wDummy;
  wDummy /= 32;
  wDummy++;
  if (HIBYTE (wDummy) == 0x01)
    chip->bcmDummyMSB = 0x40;
  else
    chip->bcmDummyMSB = 0x00;
  CW_SetDummyMSB.bByte =
    chip->bCMTTableLength | chip->bcmCCDWidthMSB |
    chip->bcmDummyMSB;
  CW_SetDummyMSB.bRegNo = 8;
  CW_SetDummy.bByte = LOBYTE (wDummy);
  CW_SetDummy.bRegNo = 11;

  RIE(usb_low_write_reg (chip, CW_SetDummyMSB));
  
  RIE(usb_low_write_reg (chip, CW_SetDummy));
  
  DBG (7, "usb_low_SetDummy: exit\n");
  return SANE_STATUS_GOOD;
}

/* A12 + A13 */
SANE_Status
usb_low_GetA12 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA12: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA12: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA12: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 12, &bPattern));

  chip->wByteWidth =
    (chip->wByteWidth & 0x3f00) + ((SANE_Word) bPattern);
  chip->wSoftResample =
    (chip->wSoftResample == 0) ? 1 : chip->wSoftResample;
  chip->pGetRow =
    (chip->wSoftResample == 1) 
    ? & usb_low_GetRowDirect : & usb_low_GetRowResample;
  chip->wRowSize = chip->wByteWidth / chip->wSoftResample;

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA12: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_GetA13 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA13: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA13: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA13: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 13, &bPattern));

  chip->wByteWidth =
    (chip->wByteWidth & 0x00ff) + (((SANE_Word) (bPattern & 0x3f)) << 8);
  chip->wSoftResample =
    (chip->wSoftResample == 0) ? 1 : chip->wSoftResample;
  chip->pGetRow =
    (chip->wSoftResample ==
     1) ? & usb_low_GetRowDirect : & usb_low_GetRowResample;
  chip->wRowSize = chip->wByteWidth / chip->wSoftResample;

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA13: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetImageByteWidth (ma1017 * chip, SANE_Word wRowSize)
{
  CONTROLWORD CW_SetLOImageByteWidth;
  CONTROLWORD CW_SetHIImageByteWidth;
  SANE_Status status;
  
  DBG (7, "usb_low_SetImageByteWidth: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetImageByteWidth: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetImageByteWidth: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->wRowSize = wRowSize;
  chip->wSoftResample =
    (chip->wSoftResample == 0) ? 1 : chip->wSoftResample;
  chip->pGetRow =
    (chip->wSoftResample ==
     1) ? &usb_low_GetRowDirect : &usb_low_GetRowResample;
  chip->wByteWidth = chip->wRowSize * chip->wSoftResample;
  if (chip->wByteWidth > 0x3fff)
    {
      DBG (3, "usb_low_SetImageByteWidth: width %d exceeded\n",
	      (int) chip->wByteWidth);
      return SANE_STATUS_INVAL;
    }

  CW_SetLOImageByteWidth.bByte = LOBYTE (chip->wByteWidth);
  CW_SetLOImageByteWidth.bRegNo = 12;

  CW_SetHIImageByteWidth.bByte = HIBYTE (chip->wByteWidth);
  CW_SetHIImageByteWidth.bRegNo = 13;

  RIE(usb_low_write_reg (chip, CW_SetLOImageByteWidth));
  
  RIE(usb_low_write_reg (chip, CW_SetHIImageByteWidth));

  DBG (7, "usb_low_SetImageByteWidth: exit\n");
  
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetSoftResample (ma1017 * chip, SANE_Word wSoftResample)
{
  CONTROLWORD CW_SetLOImageByteWidth;
  CONTROLWORD CW_SetHIImageByteWidth;
  SANE_Status status;

  DBG (7, "usb_low_SetSoftResample: start\n");
  
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetSoftResample: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetSoftResample: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  if (wSoftResample == 0x00)
    {
      DBG (3, "usb_low_SetSoftResample: wSoftResample==0\n");
      return SANE_STATUS_INVAL;
    }

  chip->wSoftResample = wSoftResample;
  chip->pGetRow =
    (chip->wSoftResample ==
     1) ? &usb_low_GetRowDirect : &usb_low_GetRowResample;
  chip->wByteWidth = chip->wRowSize * chip->wSoftResample;
  if (chip->wByteWidth > 0x3fff)
    {
      DBG (3, "usb_low_SetSoftResample: width %d exceeded",
	      (int) chip->wByteWidth);
      return SANE_STATUS_INVAL;
    }

  CW_SetLOImageByteWidth.bByte = LOBYTE (chip->wByteWidth);
  CW_SetLOImageByteWidth.bRegNo = 12;
  CW_SetHIImageByteWidth.bByte = HIBYTE (chip->wByteWidth);
  CW_SetHIImageByteWidth.bRegNo = 13;

  RIE(usb_low_write_reg (chip, CW_SetLOImageByteWidth));
  
  RIE(usb_low_write_reg (chip, CW_SetHIImageByteWidth));
  
  DBG (7, "usb_low_SetSoftResample: exit\n");
  return SANE_STATUS_GOOD;
}

/* A14 + A30W */
SANE_Status
usb_low_SetCMTLoopCount (ma1017 * chip, SANE_Word wLoopCount)
{
  CONTROLWORD CW_SetLOCMTLoopCount;
  CONTROLWORD CW_SetHICMTLoopCount;
  SANE_Status status;
  
  DBG (7, "usb_low_SetCMTLoopCount: start\n");

  CW_SetLOCMTLoopCount.bByte = LOBYTE (wLoopCount);
  CW_SetLOCMTLoopCount.bRegNo = 14;
  CW_SetHICMTLoopCount.bByte = HIBYTE (wLoopCount);
  CW_SetHICMTLoopCount.bRegNo = 30;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetCMTLoopCount: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetCMTLoopCount: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->wLoopCount = wLoopCount;

  RIE(usb_low_write_reg (chip, CW_SetLOCMTLoopCount));
  
  RIE(usb_low_write_reg (chip, CW_SetHICMTLoopCount));
  
  DBG (7, "usb_low_SetCMTLoopCount: exit\n");
  return SANE_STATUS_GOOD;
}

/* A15 */
SANE_Status
usb_low_GetA15 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA15: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA15: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA15: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 15, &bPattern));

  chip->bcmMotorEnable = bPattern & 0x80;
  chip->bcmMotorMovement = bPattern & 0x68;
  chip->bcmMotorDirection = bPattern & 10;
  chip->bcmMotorSignal = bPattern & 0x06;
  chip->bcmMotorHome = bPattern & 0x01;

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA15: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_EnableMotor (ma1017 * chip, SANE_Bool isEnable)
{
  CONTROLWORD CW_SetMotorMode;
  SANE_Status status;
  
  DBG (7, "usb_low_EnableMotor: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_EnableMotor: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_EnableMotor: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmMotorEnable = 0x00;
  if (isEnable)
    chip->bcmMotorEnable |= 0x80;
  CW_SetMotorMode.bByte =
    chip->bcmMotorEnable | chip->bcmMotorMovement |
    chip->bcmMotorDirection | chip->bcmMotorSignal |
    chip->bcmMotorHome;
  CW_SetMotorMode.bRegNo = 15;

  RIE(usb_low_write_reg (chip, CW_SetMotorMode));
  
  DBG (7, "usb_low_EnableMotor: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetMotorMovement (ma1017 * chip, SANE_Bool isFullStep,
			  SANE_Bool isDoublePhase, SANE_Bool isTwoStep)
{
  CONTROLWORD CW_SetMotorMode;
  SANE_Status status;
  
  DBG (7, "usb_low_SetMotorMovement: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetMotorMovement: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetMotorMovement: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmMotorMovement = 0x00;
  if (isFullStep)
    chip->bcmMotorMovement |= 0x40;
  if (isDoublePhase)
    chip->bcmMotorMovement |= 0x20;
  if (isTwoStep)
    chip->bcmMotorMovement |= 0x08;
  CW_SetMotorMode.bByte =
    chip->bcmMotorEnable | chip->bcmMotorMovement |
    chip->bcmMotorDirection | chip->bcmMotorSignal |
    chip->bcmMotorHome;
  CW_SetMotorMode.bRegNo = 15;

  RIE(usb_low_write_reg (chip, CW_SetMotorMode));
  
  DBG (7, "usb_low_SetMotorMovement:  exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetMotorDirection (ma1017 * chip, SANE_Bool isBackward)
{
  CONTROLWORD CW_SetMotorMode;
  SANE_Status status;
  
  DBG (7, "usb_low_SetMotorDirection: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetMotorDirection: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetMotorDirection: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmMotorDirection = 0x00;
  if (isBackward)
    chip->bcmMotorDirection |= 0x10;
  CW_SetMotorMode.bByte =
    chip->bcmMotorEnable | chip->bcmMotorMovement |
    chip->bcmMotorDirection | chip->bcmMotorSignal |
    chip->bcmMotorHome;
  CW_SetMotorMode.bRegNo = 15;

  RIE(usb_low_write_reg (chip, CW_SetMotorMode));
  
  DBG (7, "usb_low_SetMotorDirection: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetMotorSignal (ma1017 * chip, SANE_Byte bSignal)
{
  CONTROLWORD CW_SetMotorSignal;
  SANE_Status status;
  
  DBG (7, "usb_low_SetMotorSignal: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetMotorSignal: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetMotorSignal: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmMotorSignal = bSignal & 0x06;
  CW_SetMotorSignal.bByte =
    chip->bcmMotorEnable | chip->bcmMotorMovement |
    chip->bcmMotorDirection | chip->bcmMotorSignal |
    chip->bcmMotorHome;
  CW_SetMotorSignal.bRegNo = 15;

  RIE(usb_low_write_reg (chip, CW_SetMotorSignal));
  
  DBG (7, "usb_low_SetMotorSignal: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_MoveMotorHome (ma1017 * chip, SANE_Bool isHome, SANE_Bool isBackward)
{
  CONTROLWORD CW_SetMotorMode;
  SANE_Status status;

  DBG (7, "usb_low_MoveMotorHome: start\n");
  
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_MoveMotorHome: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_MoveMotorHome: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmMotorEnable = 0x00;
  chip->bcmMotorDirection = 0x00;
  chip->bcmMotorHome = 0x00;
  if (isBackward)
    chip->bcmMotorDirection |= 0x10;
  if (isHome)
    {
      chip->bcmMotorEnable |= 0x80;
      chip->bcmMotorHome |= 0x01;
    }
  CW_SetMotorMode.bByte =
    chip->bcmMotorEnable | chip->bcmMotorMovement |
    chip->bcmMotorDirection | chip->bcmMotorSignal |
    chip->bcmMotorHome;
  CW_SetMotorMode.bRegNo = 15;

  RIE(usb_low_write_reg (chip, CW_SetMotorMode));
  
  DBG (7, "usb_low_MoveMotorHome: exit\n");
  return SANE_STATUS_GOOD;
}

/* A16 */
SANE_Status
usb_low_GetA16 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA16: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA16: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA16: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 16, &bPattern));

  chip->bcmPixelDeepth = bPattern & 0xe0;
  chip->bcmImageInvert = bPattern & 0x10;
  chip->bcmOptical600 = bPattern & 0x08;
  chip->bcmSampleWay = bPattern & 0x07;

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA16: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetImageDpi (ma1017 * chip, SANE_Bool isOptical600,
		      SAMPLEWAY swSampleWay)
{
  CONTROLWORD CW_SetImageMode;
  SANE_Status status;
  
  DBG (7, "usb_low_SetImageMode: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetImageMode: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetImageMode: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmOptical600 = 0x00;
  chip->bcmSampleWay = 0x00;
  if (isOptical600)
    chip->bcmOptical600 |= 0x08;
  switch (swSampleWay)
    {
    case SW_P1P6:
      chip->bcmSampleWay = 0x01;
      break;
    case SW_P2P6:
      chip->bcmSampleWay = 0x02;
      break;
    case SW_P3P6:
      chip->bcmSampleWay = 0x03;
      break;
    case SW_P4P6:
      chip->bcmSampleWay = 0x04;
      break;
    case SW_P5P6:
      chip->bcmSampleWay = 0x05;
      break;
    case SW_P6P6:
      chip->bcmSampleWay = 0x06;
      break;
    default:
      DBG (3, "usb_low_SetImageMode: swSampleWay error\n");
      return SANE_STATUS_INVAL;
      break;
    }
  CW_SetImageMode.bByte =
    chip->bcmPixelDeepth | chip->bcmImageInvert |
    chip->bcmOptical600 | chip->bcmSampleWay;
  CW_SetImageMode.bRegNo = 16;

  RIE(usb_low_write_reg (chip, CW_SetImageMode));
  
  DBG (7, "usb_low_SetImageMode: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetPixelDeepth (ma1017 * chip, PIXELDEEPTH pdPixelDeepth)
{
  CONTROLWORD CW_SetImageMode;
  SANE_Status status;
  
  DBG (7, "usb_low_SetPixelDeepth: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetPixelDeepth: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetPixelDeepth: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmPixelDeepth = 0x00;
  switch (pdPixelDeepth)
    {
    case PD_1BIT:
      chip->bcmPixelDeepth = 0x80;
      break;
    case PD_4BIT:
      chip->bcmPixelDeepth = 0xc0;
      break;
    case PD_8BIT:
      chip->bcmPixelDeepth = 0x00;
      break;
    case PD_12BIT:
      chip->bcmPixelDeepth = 0x20;
      break;
    default:
      DBG (3, "usb_low_SetPixelDeepth: pdPixelDeepth error\n");
      return SANE_STATUS_INVAL;
    }
  CW_SetImageMode.bByte =
    chip->bcmPixelDeepth | chip->bcmImageInvert |
    chip->bcmOptical600 | chip->bcmSampleWay;
  CW_SetImageMode.bRegNo = 16;

  RIE(usb_low_write_reg (chip, CW_SetImageMode));
  
  DBG (7, "usb_low_SetPixelDeepth: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_InvertImage (ma1017 * chip, SANE_Bool isInvert)
{
  CONTROLWORD CW_SetImageMode;
  SANE_Status status;
  
  DBG (7, "usb_low_InvertImage: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_InvertImage: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_InvertImage: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmImageInvert = 0x00;
  if (isInvert)
    chip->bcmImageInvert |= 0x10;
  CW_SetImageMode.bByte =
    chip->bcmPixelDeepth | chip->bcmImageInvert |
    chip->bcmOptical600 | chip->bcmSampleWay;
  CW_SetImageMode.bRegNo = 16;

  RIE(usb_low_write_reg (chip, CW_SetImageMode));
  
  DBG (7, "usb_low_InvertImage: exit\n");
  return SANE_STATUS_GOOD;
}

/* A17 + A18 + A19 */
SANE_Status
usb_low_GetA17 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA17: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA17: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA17: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 17, &bPattern));

  chip->bRedRef = bPattern;

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA17: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_GetA18 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA18: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA18: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA18: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 18, &bPattern));

  chip->bGreenRef = bPattern;

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA18: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_GetA19 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA19: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA19: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA19:stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 19, &bPattern));

  chip->bBlueRef = bPattern;

  if (value)
    *value = bPattern;

  DBG (7, "usb_low_GetA19: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetRedRef (ma1017 * chip, SANE_Byte RedRef)
{
  CONTROLWORD CW_SetRedReference;
  SANE_Status status;
  
  DBG (7, "usb_low_SetRedRef: start\n");

  CW_SetRedReference.bByte = RedRef;
  CW_SetRedReference.bRegNo = 17;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetRedRef: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetRedRef: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bRedRef = RedRef;

  RIE(usb_low_write_reg (chip, CW_SetRedReference));
  
  DBG (7, "usb_low_SetRedRef: stop\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetGreenRef (ma1017 * chip, SANE_Byte GreenRef)
{
  CONTROLWORD CW_SetGreenReference;
  SANE_Status status;
  
  DBG (7, "usb_low_GreenRef: start\n");

  CW_SetGreenReference.bByte = GreenRef;
  CW_SetGreenReference.bRegNo = 18;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GreenRef: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GreenRef: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bGreenRef = GreenRef;

  RIE(usb_low_write_reg (chip, CW_SetGreenReference));
  
  DBG (7, "usb_low_GreenRef: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetBlueRef (ma1017 * chip, SANE_Byte BlueRef)
{
  CONTROLWORD CW_SetBlueReference;
  SANE_Status status;
  
  DBG (7, "usb_low_BlueRef: start\n");
      
  CW_SetBlueReference.bByte = BlueRef;
  CW_SetBlueReference.bRegNo = 19;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_BlueRef: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_BlueRef: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bBlueRef = BlueRef;

  RIE(usb_low_write_reg (chip, CW_SetBlueReference));
  
  DBG (7, "usb_low_BlueRef: stop\n");
  return SANE_STATUS_GOOD;
}

/* A20 + A21 + A22 */
SANE_Status
usb_low_GetA20 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA20: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA20: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA20: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }
  RIE(usb_low_read_reg (chip, 20, &bPattern));

  chip->bRedPD = bPattern;

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA20: stop\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_GetA21 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA21: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA21: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA21: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 21, &bPattern));

  chip->bGreenPD = bPattern;

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA21: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_GetA22 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA22: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA22: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA22: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 22, &bPattern));

  chip->bBluePD = bPattern;

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA22: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetRedPD (ma1017 * chip, SANE_Byte RedPD)
{
  CONTROLWORD CW_SetRedPowerDelay;
  SANE_Status status;
  
  DBG (7, "usb_low_SetRedPD: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetRedPD: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetRedPD: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bRedPD = RedPD;
  CW_SetRedPowerDelay.bByte = chip->bRedPD;
  CW_SetRedPowerDelay.bRegNo = 20;

  RIE(usb_low_write_reg (chip, CW_SetRedPowerDelay));
  
  DBG (7, "usb_low_SetRedPD: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetGreenPD (ma1017 * chip, SANE_Byte GreenPD)
{
  CONTROLWORD CW_SetGreenPowerDelay;
  SANE_Status status;
  
  DBG (7, "usb_low_SetGreenPD: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetGreenPD: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetGreenPD: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bGreenPD = GreenPD;
  CW_SetGreenPowerDelay.bByte = chip->bGreenPD;
  CW_SetGreenPowerDelay.bRegNo = 21;

  RIE(usb_low_write_reg (chip, CW_SetGreenPowerDelay));
  
  DBG (7, "usb_low_SetGreenPD: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetBluePD (ma1017 * chip, SANE_Byte BluePD)
{
  CONTROLWORD CW_SetBluePowerDelay;
  SANE_Status status;
  
  DBG (7, "usb_low_SetBluePD: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetBluePD: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetBluePD: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bBluePD = BluePD;
  CW_SetBluePowerDelay.bByte = chip->bBluePD;
  CW_SetBluePowerDelay.bRegNo = 22;

  RIE(usb_low_write_reg (chip, CW_SetBluePowerDelay));
  
  DBG (7, "usb_low_SetBluePD: exit\n");
  return SANE_STATUS_GOOD;
}

/* A23 */
SANE_Status
usb_low_GetA23 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA23: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA23: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA23: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 23, &bPattern));

  chip->bcmA23 = bPattern;

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA23: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_TurnPeripheralPower (ma1017 * chip, SANE_Bool isOn)
{
  CONTROLWORD CW_TurnPeripheralPower;
  SANE_Status status;
  
  DBG (7, "usb_low_TurnPeripheralPower: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_TurnPeripheralPower: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_TurnPeripheralPower: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmA23 &= 0x7f;
  if (isOn)
    chip->bcmA23 |= 0x80;
  CW_TurnPeripheralPower.bByte = chip->bcmA23;
  CW_TurnPeripheralPower.bRegNo = 23;

  RIE(usb_low_write_reg (chip, CW_TurnPeripheralPower));
  
  DBG (7, "usb_low_TurnPeripheralPower: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_TurnLampPower (ma1017 * chip, SANE_Bool isOn)
{
  CONTROLWORD CW_TurnLampPower;
  SANE_Status status;

  DBG (7, "usb_low_TurnLampPower: start\n");
      
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_TurnLampPower: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_TurnLampPower: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmA23 &= 0xbf;
  if (isOn)
    chip->bcmA23 |= 0x40;
  CW_TurnLampPower.bByte = chip->bcmA23;
  CW_TurnLampPower.bRegNo = 23;

  RIE(usb_low_write_reg (chip, CW_TurnLampPower));
  
  DBG (7, "usb_low_TurnLampPower: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetIO3 (ma1017 * chip, SANE_Bool isHigh)
{
  CONTROLWORD CW_SetIO3;
  SANE_Status status;
  
  DBG (7, "usb_low_SetIO3: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetIO3: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetIO3: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmA23 &= 0xf7;
  if (isHigh)
    chip->bcmA23 |= 0x08;
  CW_SetIO3.bByte = chip->bcmA23;
  CW_SetIO3.bRegNo = 23;

  RIE(usb_low_write_reg (chip, CW_SetIO3));
  
  DBG (7, "usb_low_SetIO3: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetLEDLightAll (ma1017 * chip, SANE_Bool isLightAll)
{
  CONTROLWORD CW_SetLEDLightAll;
  SANE_Status status;
  
  DBG (7, "usb_low_SetLEDLightAll: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetLEDLightAll: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetLEDLightAll: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmA23 &= 0xfe;
  if (isLightAll)
    chip->bcmA23 |= 0x01;
  CW_SetLEDLightAll.bByte = chip->bcmA23;
  CW_SetLEDLightAll.bRegNo = 23;

  RIE(usb_low_write_reg (chip, CW_SetLEDLightAll));
  
  DBG (7, "usb_low_SetLEDLightAll: exit\n");
  return SANE_STATUS_GOOD;
}

/* A24 */
SANE_Status
usb_low_GetA24 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA24: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA24: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA24: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 24, &bPattern));

  chip->bcmFy1Delay = bPattern & 0x01;
  chip->bcmSpecialAD = bPattern & 0x02;

  if (value)
    *value = bPattern;
  
  DBG (7, "usb_low_GetA24: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetADTiming (ma1017 * chip, SANE_Byte bPattern)
{
  CONTROLWORD CW_SetADTiming;
  SANE_Status status;
  
  DBG (7, "usb_low_SetADTiming: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetADTiming: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetADTiming: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmFy1Delay = bPattern & 0x01;
  chip->bcmSpecialAD = bPattern & 0x02;

  CW_SetADTiming.bByte = chip->bcmSpecialAD | chip->bcmFy1Delay;
  CW_SetADTiming.bRegNo = 24;

  RIE(usb_low_write_reg (chip, CW_SetADTiming));
  
  DBG (7, "usb_low_SetADTiming: exit\n");
  return SANE_STATUS_GOOD;
}

/* A25 + A26 */
SANE_Status
usb_low_SetSerialByte1 (ma1017 * chip, SANE_Byte Byte1)
{
  CONTROLWORD CW_SetSerialByte1;
  SANE_Status status;
  
  DBG (7, "usb_low_SetSerialByte1: start\n");

  CW_SetSerialByte1.bByte = Byte1;
  CW_SetSerialByte1.bRegNo = 25;


  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetSerialByte1: not opened\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetSerialByte1: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_write_reg (chip, CW_SetSerialByte1));
  
  DBG (7, "usb_low_SetSerialByte1: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetSerialByte2 (ma1017 * chip, SANE_Byte Byte2)
{
  CONTROLWORD CW_SetSerialByte2;
  SANE_Status status;
  
  DBG (7, "usb_low_SetSerialByte2: start\n");

  CW_SetSerialByte2.bByte = Byte2;
  CW_SetSerialByte2.bRegNo = 26;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetSerialByte2: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetSerialByte2: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_write_reg (chip, CW_SetSerialByte2));
  
  DBG (7, "usb_low_SetSerialByte2: exit\n");
  return SANE_STATUS_GOOD;
}

/* A27 */
SANE_Status
usb_low_GetA27 (ma1017 * chip, SANE_Byte *value)
{
  SANE_Byte bPattern;
  SANE_Status status;

  DBG (7, "usb_low_GetA27: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetA27: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetA27: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 27, &bPattern));

  chip->bcmSCLK = bPattern & 0x80;
  chip->bcmSEN = bPattern & 0x40;
  chip->bSerialLength = bPattern & 0x1f;

  if (value)
    *value = bPattern;

  DBG (7, "usb_low_GetA27: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_SetSerialFormat (ma1017 * chip, SANE_Byte Format)
{
  CONTROLWORD CW_SetSerialFormat;
  SANE_Status status;
  
  DBG (7, "usb_low_SetSerialFormat: start\n");

  CW_SetSerialFormat.bByte = Format;
  CW_SetSerialFormat.bRegNo = 27;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_SetSerialFormat: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_SetSerialFormat: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  chip->bcmSCLK = Format & 0x80;
  chip->bcmSEN = Format & 0x40;
  chip->bSerialLength = Format & 0x1f;

  RIE(usb_low_write_reg (chip, CW_SetSerialFormat));
  
  DBG (7, "usb_low_SetSerialFormat: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_GetHomeSensor (ma1017 * chip)
{
  SANE_Byte Byte;
  SANE_Status status;

  DBG (7, "usb_low_GetHomeSensor: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_GetHomeSensor: not opened yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_GetHomeSensor: stop rowing first\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_read_reg (chip, 31, &Byte));

  DBG (7, "usb_low_GetHomeSensor: exit\n");
  if ((Byte & 0x80) != 0)
    return SANE_STATUS_GOOD;
  else
    return SANE_STATUS_IO_ERROR;
}

/* Special Mode */
SANE_Status
usb_low_StartRowing (ma1017 * chip)
{
  SANE_Word dwLineOfFirst = 0;
  SANE_Word dwLineOfSecond = 0;
  int i;
  SANE_Status status;
  
  DBG (7, "usb_low_StartRowing: start\n");

  if (chip->wLoopCount == 0)
    {
      DBG (3, "usb_low_StartRowing wLoopCount hasn't been set yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->dwCMTTableLength == 0)
    {
      DBG (3, "usb_low_StartRowing: dwCMTTableLength hasn't been set yet\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->dwCMTTableLength <= chip->dwCMTSecondPos)
    {
      DBG (3, "usb_low_StartRowing: dwCMTSecondPos cannot be larger than "
	   "dwCMTTableLength\n");
      return SANE_STATUS_INVAL;
    }

  for (i = 0; i < (int) chip->dwCMTSecondPos; i++)
    {
      if (chip->isTransferTable[i])
	dwLineOfFirst++;
    }
  for (; i < (int) chip->dwCMTTableLength; i++)
    {
      if (chip->isTransferTable[i])
	{
	  dwLineOfFirst++;
	  dwLineOfSecond++;
	}
    }

  chip->dwTotalLine =
    ((SANE_Word) (chip->wLoopCount - 1)) * dwLineOfSecond + dwLineOfFirst;
  chip->dwLeftLine = chip->dwTotalLine;

  RIE(usb_low_StartCMTTable (chip));

  DBG (7, "usb_low_StartRowing: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_StopRowing (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (7, "usb_low_StopRowing: start\n");

  RIE(usb_low_StopCMTTable (chip));

  DBG (7, "usb_low_StopRowing: exit\n");
  return SANE_STATUS_GOOD;
 
}

SANE_Status
usb_low_WaitRowingStop (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (7, "usb_low_WaitRowingStop: start\n");
  if (chip->dwTotalLine != 0)
    {
      DBG (3, "usb_low_WaitRowingStop: dwTotalLine must be 0\n");
      return SANE_STATUS_INVAL;
    }

  RIE(usb_low_wait_rowing (chip));
  chip->isRowing = SANE_FALSE;
  DBG (7, "usb_low_WaitRowingStop: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_read_all_registers (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (7, "usb_low_read_all_registers: start\n");

  RIE(usb_low_GetA2 (chip, 0));
  RIE(usb_low_GetA4 (chip, 0));
  RIE(usb_low_GetA6 (chip, 0));
  RIE(usb_low_GetA7 (chip, 0));
  RIE(usb_low_GetA8 (chip, 0));
  RIE(usb_low_GetA9 (chip, 0));
  RIE(usb_low_GetA10 (chip, 0));
  RIE(usb_low_GetA11 (chip, 0));
  RIE(usb_low_GetA12 (chip, 0));
  RIE(usb_low_GetA13 (chip, 0));
  RIE(usb_low_GetA15 (chip, 0));
  RIE(usb_low_GetA16 (chip, 0));
  RIE(usb_low_GetA17 (chip, 0));
  RIE(usb_low_GetA18 (chip, 0));
  RIE(usb_low_GetA19 (chip, 0));
  RIE(usb_low_GetA20 (chip, 0));
  RIE(usb_low_GetA21 (chip, 0));
  RIE(usb_low_GetA22 (chip, 0));
  RIE(usb_low_GetA23 (chip, 0));
  RIE(usb_low_GetA24 (chip, 0));
  RIE(usb_low_GetA27 (chip, 0));
  
  return SANE_STATUS_GOOD;
  DBG (7, "usb_low_read_all_registers: exit\n");
}

SANE_Status
usb_low_SetCIS (ma1017 * chip, SENSOR Sensor)
{
  chip->Sensor = Sensor;
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_GetRow (ma1017 * chip, SANE_Byte * pData, SANE_Word *LeftLine)
{
  SANE_Status status;
  
  DBG (7, "usb_low_GetRow: start\n");
  RIE((*chip->pGetRow) (chip, pData, LeftLine));
  DBG (7, "usb_low_GetRow: exit\n");
  return SANE_STATUS_GOOD;;
}

SANE_Status
usb_low_GetRowDirect (ma1017 * chip, SANE_Byte * pData, SANE_Word *LeftLine)
{
  SANE_Status status;
  
  DBG (7, "usb_low_GetRowDirect: start\n");
  if (chip->dwLeftLine == 0)
    {
      DBG (3, "usb_low_GetRowDirect: dwLeftLine == 0\n");
      return SANE_STATUS_INVAL;
    }

  if (chip->dwLeftLine <= 1)
    {
      RIE(usb_low_read_rows (chip, pData, chip->wByteWidth));
      RIE(usb_low_wait_rowing (chip));

      chip->dwLeftLine = 0x00;
      chip->isRowing = SANE_FALSE;
      *LeftLine = 0;
    }
  else
    {
      RIE(usb_low_read_rows (chip, pData, chip->wByteWidth));
      chip->dwLeftLine--;
      *LeftLine = chip->dwLeftLine;
    }
  DBG (7, "usb_low_GetRowDirect: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_GetRowResample (ma1017 * chip, SANE_Byte * pData, SANE_Word *LeftLine)
{
  static SANE_Byte s_ResampleBuffer[8 * 1024];
  SANE_Word *pPixelTemp;
  SANE_Word i;
  SANE_Word j;
  SANE_Word k;
  SANE_Status status;
  
  DBG (7, "usb_low_GetRowResample: start\n");

  if (chip->dwLeftLine == 0)
    {
      DBG (3, "usb_low_GetRowResample: dwLeftLine == 0\n");
      return SANE_STATUS_INVAL;
    }

  if (chip->dwLeftLine <= 1)
    {
      RIE(usb_low_read_rows (chip, s_ResampleBuffer, chip->wByteWidth));

      if ((chip->Sensor == SR_CANON600) && (chip->bcmPixelDeepth == 0x20))
	{
	  pPixelTemp = (SANE_Word *) malloc (6 * 1024 * sizeof (SANE_Word));

	  j = 0;
	  for (i = 0; i < chip->wByteWidth; i += 3)
	    {
	      pPixelTemp[j] = (SANE_Word) s_ResampleBuffer[i];
	      pPixelTemp[j] |= ((SANE_Word) 
				(s_ResampleBuffer[i + 1] & 0xf0)) << 4;
	      j++;
	      pPixelTemp[j] = ((SANE_Word)
			       (s_ResampleBuffer[i + 1] & 0x0f)) << 8;
	      pPixelTemp[j] |= (SANE_Word) s_ResampleBuffer[i + 2];
	      j++;
	    }

	  k = 0;
	  for (i = 0; i < j; i += chip->wSoftResample * 2)
	    {
	      pData[k] = (SANE_Byte) (pPixelTemp[i] & 0x00ff);
	      k++;
	      pData[k] = (SANE_Byte) ((pPixelTemp[i] & 0x0f00) >> 4);
	      pData[k] |= (SANE_Byte) ((pPixelTemp[i + 2] & 0x0f00) >> 8);
	      k++;
	      pData[k] = (SANE_Byte) (pPixelTemp[i + 2] & 0x00ff);
	      k++;
	    }
	  free (pPixelTemp);
	}
      else /* fixme ?*/
	{
	  for (i = 0; i < chip->wByteWidth; i += chip->wSoftResample)
	    *(pData++) = s_ResampleBuffer[i];
	}
      RIE(usb_low_wait_rowing (chip));

      chip->dwLeftLine = 0x00;
      chip->isRowing = SANE_FALSE;
      *LeftLine = 0;
    }
  else
    {
      RIE(usb_low_read_rows (chip, s_ResampleBuffer, chip->wByteWidth));
      
      if ((chip->Sensor == SR_CANON600) && (chip->bcmPixelDeepth == 0x20))
	{
	  pPixelTemp = (SANE_Word *) malloc (6 * 1024 * sizeof (SANE_Word));
	  j = 0;
	  for (i = 0; i < chip->wByteWidth; i += 3)
	    {
	      pPixelTemp[j] = (SANE_Word) s_ResampleBuffer[i];
	      pPixelTemp[j] |= 
		((SANE_Word) (s_ResampleBuffer[i + 1] & 0xf0)) << 4;
	      j++;
	      pPixelTemp[j] = 
		((SANE_Word) (s_ResampleBuffer[i + 1] & 0x0f)) << 8;
	      pPixelTemp[j] |= (SANE_Word) s_ResampleBuffer[i + 2];
	      j++;
	    }

	  k = 0;
	  for (i = 0; i < j; i += chip->wSoftResample * 2)
	    {
	      pData[k] = (SANE_Byte) (pPixelTemp[i] & 0x00ff);
	      k++;
	      pData[k] = (SANE_Byte) ((pPixelTemp[i] & 0x0f00) >> 4);
	      pData[k] |= (SANE_Byte) ((pPixelTemp[i + 2] & 0x0f00) >> 8);
	      k++;
	      pData[k] = (SANE_Byte) (pPixelTemp[i + 2] & 0x00ff);
	      k++;
	    }
	  free (pPixelTemp);
	}
      else /* fixme? */
	{
	  for (i = 0; i < chip->wByteWidth; i += chip->wSoftResample)
	    *(pData++) = s_ResampleBuffer[i];
	}
      chip->dwLeftLine--;
      *LeftLine = chip->dwLeftLine;
    }
  DBG (7, "usb_low_GetRowResample: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_wait_rowing (ma1017 * chip)
{
  SANE_Byte ReadByte;
  size_t n;
  SANE_Status status;

  DBG (7, "usb_low_wait_rowing: start\n");

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_wait_rowing: open first\n");
      return SANE_STATUS_INVAL;
    }
  if (!chip->isRowing)
    {
      DBG (3, "usb_low_wait_rowing: not rowing\n");
      return SANE_STATUS_INVAL;
    }
  n = 1;
  status = sanei_usb_read_bulk (chip->fd, (SANE_Byte *) &ReadByte, &n);
  if (status != SANE_STATUS_GOOD || n != 1)
    {
      DBG (3, "usb_low_wait_rowing: couldn't read: %s\n",
	   sane_strstatus (status));
      return SANE_STATUS_IO_ERROR;
    }
  chip->isRowing = SANE_FALSE;
  DBG (7, "usb_low_wait_rowing: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_read_rows (ma1017 * chip, SANE_Byte * pData, SANE_Word dwByteCount)
{
  size_t n;
  SANE_Status status;

  DBG (7, "usb_low_read_rows: start\n");
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_read_rows: isOpened==SANE_FALSE\n");
      return SANE_STATUS_INVAL;
    }
  if (!chip->isRowing)
    {
      DBG (3, "usb_low_read_rows: isRowing==SANE_FALSE\n");
      return SANE_STATUS_INVAL;
    }

  n = dwByteCount;
  status = sanei_usb_read_bulk (chip->fd, (SANE_Byte *) pData, &n);
  if (status != SANE_STATUS_GOOD)
    {
      DBG (7, "usb_low_read_rows: problems during read: %s\n",
	   sane_strstatus (status));
      return status;
    }

  if ((SANE_Word) n != dwByteCount)
    {
      DBG (7, "usb_low_read_rows: problems during read, wanted %ul, got %ul "
	   "bytes: %s\n", (unsigned long) dwByteCount, (unsigned long) n,
	   sane_strstatus (status));
      return SANE_STATUS_IO_ERROR;
    }

  DBG (7, "usb_low_read_rows: exit, read %ul bytes\n", (unsigned long) n);
  return SANE_STATUS_GOOD;
}


SANE_Status
usb_low_write_reg (ma1017 * chip, CONTROLWORD Control)
{
  size_t n;
  SANE_Status status;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_write_reg: open first\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_write_reg: rowing, stop first\n");
      return SANE_STATUS_INVAL;
    }
  if (Control.bRegNo > 0x20)
    {
      DBG (3, "usb_low_write_reg: RegNo out of range\n");
      return SANE_STATUS_INVAL;
    }
  n = sizeof (CONTROLWORD);
  status = sanei_usb_write_bulk (chip->fd, (SANE_Byte *) &Control, &n);
  if (status != SANE_STATUS_GOOD || n != sizeof (CONTROLWORD))
    {
      DBG (3, "usb_low_write_reg: couldn't write, tried to write %ul, "
	   "wrote %ul: %s\n", (unsigned long) sizeof (CONTROLWORD), 
	   (unsigned long) n, sane_strstatus (status));
      return SANE_STATUS_IO_ERROR;
    }
  DBG (7, "usb_low_write_reg: reg: %02x, value: %02x\n",
       Control.bRegNo, Control.bByte);
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_read_reg (ma1017 * chip, SANE_Byte RegNo, SANE_Byte *data)
{
  TWOPLY TP_ReadReg;
  SANE_Byte ReadByte;
  size_t n;
  SANE_Status status;

  TP_ReadReg.bByte = 0x00;
  TP_ReadReg.bRegNo = RegNo | 0x20;

  if (!chip->isOpened)
    {
      DBG (3, "usb_low_read_reg: open first\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isRowing)
    {
      DBG (3, "usb_low_read_reg: rowing, stop first\n");
      return SANE_STATUS_INVAL;
    }
  if (RegNo > 0x20)
    {
      DBG (3, "usb_low_read_reg: RegNo out of range\n");
      return SANE_STATUS_INVAL;
    }
  n = sizeof (TWOPLY);
  status = sanei_usb_write_bulk (chip->fd, (SANE_Byte *) &TP_ReadReg,
				 &n);
  if (status != SANE_STATUS_GOOD || n != sizeof (TWOPLY))
    {
      DBG (3, "usb_low_read_reg: couldn't write, tried to write %ul, "
	   "wrote %ul: %s\n", (unsigned long) sizeof (CONTROLWORD), 
	   (unsigned long) n, sane_strstatus (status));
      return SANE_STATUS_IO_ERROR;
    }
  n = 1;
  status = sanei_usb_read_bulk (chip->fd, (SANE_Byte *) &ReadByte, &n);
  if (status != SANE_STATUS_GOOD || n != 1)
    {
      DBG (3, "usb_low_read_reg: couldn't read, tried to read %ul, "
	   "wrote %ul: %s\n", (unsigned long) 1, 
	   (unsigned long) n, sane_strstatus (status));
      return SANE_STATUS_IO_ERROR;
    }
  if (data)
    *data = ReadByte;
  DBG (7, "usb_low_read_reg: Reg: %02x, Value: %02x\n",
       RegNo, ReadByte);
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_identify_scanner (SANE_Int fd, Mustek_Type *scanner_type)
{
  SANE_Status status;
  SANE_Word devvendor, devproduct;
  Mustek_Type devtype;

  DBG (7, "usb_low_identify_scanner: start\n");

  status = sanei_usb_get_vendor_product (fd, &devvendor, &devproduct);
  devtype = mustek_usb_type_unknown;
  if (status == SANE_STATUS_GOOD)
    {
      if (devvendor == 0x055f)
	{
	  switch (devproduct)
	    {
	    case 0x0001: 
	      devtype = mustek_usb_type_1200cu;
	      break;
	    case 0x0002: 
	      devtype = mustek_usb_type_600cu;
	      break;
#if 0	      
	    case 0x0003:
	      devtype = mustek_usb_type_1200cu;
	      break;
#endif
	    case 0x0006:
	      devtype = mustek_usb_type_1200ub;
	      break;
	    case 0x0008:
	      devtype = mustek_usb_type_1200cu_plus;
	      break;
	    default:
	      if (scanner_type)
		*scanner_type = devtype;
	      DBG (3, "usb_low_identify_scanner: unknown product id: "
		   "%d\n", devproduct);
	      return SANE_STATUS_INVAL;
	      break;
	    }
	}
      else
	{
	  if (scanner_type)
	    *scanner_type = devtype;
	  DBG (3, "usb_low_identify_scanner: unknown vendor id: %d\n",
	       devvendor);
	  return SANE_STATUS_INVAL;
	}
    }
  if (scanner_type)
    *scanner_type = devtype;
  DBG (7, "usb_low_identify_scanner: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_Open (ma1017 * chip, SANE_String_Const devname)
{
  SANE_Status status;
  Mustek_Type scanner_type;

  DBG (7, "usb_low_Open: start: chip = %p\n", chip);

  if (chip->isRowing)
    {
      DBG (3, "usb_low_Open: already rowing\n");
      return SANE_STATUS_INVAL;
    }
  if (chip->isOpened)
    {
      DBG (3, "usb_low_Open: already opened\n");
      return SANE_STATUS_INVAL;
    }

  status = sanei_usb_open ((SANE_String_Const) devname, &chip->fd);
  
  if (status == SANE_STATUS_GOOD)
    {
      DBG (7, "usb_low_Open: device %s successfully opened\n", devname);
      chip->isOpened = SANE_TRUE;
      /* Try to get vendor and device ids */
      DBG(7, "usb_low_Open: trying to identify device `%s'\n", devname);
      RIE(usb_low_identify_scanner (chip->fd, &scanner_type));
      if (status != SANE_STATUS_GOOD)
	{
	  DBG (3, "usb_low_Open: device `%s' doesn't look like a supported "
	       "scanner\n", devname);
	  sanei_usb_close (chip->fd);
	  return status;
	}
      else
	{
	  if (scanner_type == mustek_usb_type_unknown)
	    {
	      DBG (3, "usb_low_Open: device `%s' can't be identified\n",
		   devname);
	    }
	  else if (scanner_type != chip->scanner_type)
	    {
	      DBG (3, "usb_low_Open: device `%s' is supported but"
		   "it's not the same as at the start\n", devname);
	      return SANE_STATUS_INVAL;
	    }
	}
    }
  else
    {
      DBG (1, "usb_low_Open: device %s couldn't be opened: %s\n", 
	   devname, sane_strstatus (status));
      return status;
    }

  chip->isOpened = SANE_TRUE;

  RIE(usb_low_read_all_registers (chip));

  DBG (7, "usb_low_Open: exit, type is %d\n", scanner_type);
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_low_Close (ma1017 * chip)
{
  DBG (7, "usb_low_Close: start, chip=%p\n", chip);
  if (!chip->isOpened)
    {
      DBG (3, "usb_low_Close: already close or never opened\n");
      return SANE_STATUS_INVAL;
    }

  if (chip->fd >= 0)
    {
      if (chip->isRowing)
	usb_low_StopRowing (chip);
      sanei_usb_close (chip->fd);
      chip->fd = -1;
    }
  chip->isOpened = SANE_FALSE;
  chip->isRowing = SANE_FALSE;

  DBG (7, "usb_low_Close: exit\n");
  return SANE_STATUS_GOOD;
}
