/* sane - Scanner Access Now Easy.

   Copyright (C) 2000 Mustek.
   Maintained by Tom Wang <tom.wang@mustek.com.tw>

   Updates (C) 2001 by Henning Meier-Geinitz.

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB flatbed scanners.  */

#ifndef mustek_usb_low_h
#define mustek_usb_low_h

#include "../include/sane/sane.h"


/* ---------------------------------- macros ------------------------------ */


/* calculate the minimum/maximum values */
#define min(a, b) (((a) < (b)) ? (a) : (b))
#define max(a, b) (((a) > (b)) ? (a) : (b))
/* return the lower/upper 8 bits of a 16 bit word */
#define HIBYTE(w) ((SANE_Byte)(((SANE_Word)(w) >> 8) & 0xFF))
#define LOBYTE(w) ((SANE_Byte)(w))
/* RIE: return if error */
#define RIE(function) do {status = function; if (status != SANE_STATUS_GOOD) \
                        return status;} while (SANE_FALSE)


/* ---------------------------------- types ------------------------------- */


typedef enum Mustek_Type
{
  mustek_usb_type_unknown = 0,
  mustek_usb_type_1200ub,
  mustek_usb_type_1200cu,
  mustek_usb_type_1200cu_plus,
  mustek_usb_type_600cu
}
Mustek_Type;

typedef enum tagSENSOR 
{
  SR_NONE=0,
  SR_INI=1,
  SR_INI_DARK=2,
  SR_CANON300=3,
  SR_CANON600=4,
  SR_TOSHIBA600=5,
  SR_CANON300600=6
} 
SENSOR;

typedef struct tagTWOPLY
{
  SANE_Byte bByte;
  SANE_Byte bRegNo;
}
TWOPLY, *PTWOPLY;

typedef struct tagCONTROLWORD
{
  SANE_Byte bByte;
  SANE_Byte bRegNo;
}
CONTROLWORD, *PCONTROLWORD;

struct ma1017;

typedef struct ma1017
{
  int fd;

  SANE_Bool isOpened;
  SANE_Bool isRowing;

  /* A2 */
  SANE_Byte bcmAppend;
  SANE_Byte bcmTestSRAM;
  SANE_Byte bcmFixPattern;
  /* A4 */
  SANE_Byte bcmSelect;
  SANE_Byte bcmFrontEnd;
  /* A6 */
  SANE_Byte bcmRGBSelPin;
  SANE_Byte bcmASICIoPins;
  /* A7 */
  SANE_Byte bcmTiming;
  SANE_Byte bcmSRAMBank;
  /* A8 */
  SANE_Byte bcmDummyMSB;
  SANE_Byte bcmCCDWidthMSB;
  SANE_Byte bCMTTableLength;
  /* A9 */
  SANE_Byte bCMTSecondPos;
  /* A10 + A8ID5 */
  SANE_Word wCCDWidth;
  /* A11 + A8ID6 */
  SANE_Word wDummy;
  /* A12 + A13 */
  SANE_Word wByteWidth;
  /* A14 + A30W */
  SANE_Word wLoopCount;
  /* A15 */
  SANE_Byte bcmMotorEnable;
  SANE_Byte bcmMotorMovement;
  SANE_Byte bcmMotorDirection;
  SANE_Byte bcmMotorSignal;
  SANE_Byte bcmMotorHome;
  /* A16 */
  SANE_Byte bcmPixelDeepth;
  SANE_Byte bcmImageInvert;
  SANE_Byte bcmOptical600;
  SANE_Byte bcmSampleWay;
  /* A17 + A18 + A19 */
  SANE_Byte bRedRef;
  SANE_Byte bGreenRef;
  SANE_Byte bBlueRef;
  /* A20 + A21 + A22 */
  SANE_Byte bRedPD;
  SANE_Byte bGreenPD;
  SANE_Byte bBluePD;
  /* A23 */
  SANE_Byte bcmA23;
  /* A24 */
  SANE_Byte bcmFy1Delay;
  SANE_Byte bcmSpecialAD;
  /* A27 */
  SANE_Byte bcmSCLK;
  SANE_Byte bcmSEN;
  SANE_Byte bSerialLength;

  /* Use for Rowing */
  SANE_Status (*pGetRow) (struct ma1017 * chip, SANE_Byte * pRow, 
			  SANE_Word *LeftLine);

  SANE_Word dwCMTTableLength;
  SANE_Word dwCMTSecondPos;
  SANE_Word wRowSize;
  SANE_Word wSoftResample;
  SANE_Word dwTotalLine;
  SANE_Word dwLeftLine;
  SANE_Bool isTransferTable[32];
  SENSOR Sensor;
  Mustek_Type scanner_type;
}
ma1017;

typedef enum tagCHANNEL
{
  CH_NONE = 0,
  CH_RED = 1,
  CH_GREEN = 2,
  CH_BLUE = 3
}
CHANNEL;

typedef int TIMING;

typedef enum tagBANKSIZE
{
  BS_NONE = 0,
  BS_4K = 1,
  BS_8K = 2,
  BS_16K = 3
}
BANKSIZE;

typedef enum tagPIXELDEEPTH
{
  PD_NONE = 0,
  PD_1BIT = 1,
  PD_4BIT = 2,
  PD_8BIT = 3,
  PD_12BIT = 4
}
PIXELDEEPTH;

typedef enum tagSAMPLEWAY
{
  SW_NONE = 0,
  SW_P1P6 = 1,
  SW_P2P6 = 2,
  SW_P3P6 = 3,
  SW_P4P6 = 4,
  SW_P5P6 = 5,
  SW_P6P6 = 6
}
SAMPLEWAY;

typedef enum tagFLASHSPEED
{
  FS_NONE = 0,
  FS_LOWEST = 1,
  FS_LOW = 2,
  FS_HIGH = 3,
  FS_HIGHEST = 4
}
FLASHSPEED;


/* ------------------------- function declarations ------------------------ */


static SANE_Status usb_low_init (ma1017 ** chip);
static SANE_Status usb_low_exit (ma1017 * chip);
/* A0 ~ A1 */
static SANE_Status usb_low_SetCMTTable (ma1017 * chip, int iIndex,
					CHANNEL chChannel,
					SANE_Bool isMoveMotor,
					SANE_Bool isTransfer);
/* A2 */
static SANE_Status usb_low_GetA2 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_StartCMTTable (ma1017 * chip);
static SANE_Status usb_low_StopCMTTable (ma1017 * chip);
static SANE_Status usb_low_SetTestSRAMMode (ma1017 * chip, SANE_Bool isTest);
static SANE_Status usb_low_SetFixPattern (ma1017 * chip, SANE_Bool isFix);
/* A3 */
static SANE_Status usb_low_AdjustTiming (ma1017 * chip,
					 SANE_Byte bAdjustPattern);
/* A4 */
static SANE_Status usb_low_GetA4 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SelectTiming (ma1017 * chip,
					 SANE_Byte bSelectPattern);
static SANE_Status usb_low_TurnFrontEndMode (ma1017 * chip, SANE_Bool isOn);
/* A6 */
static SANE_Status usb_low_GetA6 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetASICIoPins (ma1017 * chip, 
					  SANE_Byte bIoPinPattern);
static SANE_Status usb_low_SetRGBSelPins (ma1017 * chip,
					  SANE_Byte bRGBSelPinPattern);
/* A7 */
static SANE_Status usb_low_GetA7 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetTiming (ma1017 * chip, SANE_Byte bTimingPattern);
static SANE_Status usb_low_SetSRAMBank (ma1017 * chip, BANKSIZE bsBankSize);
/* A8 */
static SANE_Status usb_low_GetA8 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetCMTTableLength (ma1017 * chip, 
					      SANE_Byte bLength);
/* A9 */
static SANE_Status usb_low_GetA9 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetCMTSecondPosition (ma1017 * chip,
						 SANE_Byte bPosition);
/* A10 + A8ID5 */
static SANE_Status usb_low_GetA10 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetCCDWidth (ma1017 * chip, SANE_Word wCCDWidth);
/* A11 + A8ID6 */
static SANE_Status usb_low_GetA11 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetDummy (ma1017 * chip, SANE_Word Dummy);
/* A12 + A13 */
static SANE_Status usb_low_GetA12 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_GetA13 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetImageByteWidth (ma1017 * chip,
					      SANE_Word wRowSize);
static SANE_Status usb_low_SetSoftResample (ma1017 * chip,
					    SANE_Word wSoftResample);
/* A14 + A30W */
static SANE_Status usb_low_SetCMTLoopCount (ma1017 * chip,
					    SANE_Word LoopCount);
/* A15 */
static SANE_Status usb_low_GetA15 (ma1017 * chip, SANE_Byte *value);

static SANE_Status usb_low_EnableMotor (ma1017 * chip, SANE_Bool isEnable);
static SANE_Status usb_low_SetMotorMovement (ma1017 * chip,
					     SANE_Bool isFullStep,
					     SANE_Bool isDoublePhase,
					     SANE_Bool isTwoStep);
static SANE_Status usb_low_SetMotorSignal (ma1017 * chip, SANE_Byte bSignal);
static SANE_Status usb_low_SetMotorDirection (ma1017 * chip,
					      SANE_Bool isBackward);
static SANE_Status usb_low_MoveMotorHome (ma1017 * chip, SANE_Bool isHome,
					  SANE_Bool isBackward);
/* A16 */
static SANE_Status usb_low_GetA16 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetImageDpi (ma1017 * chip,
					SANE_Bool isOptical600,
					SAMPLEWAY swSampleWay);
static SANE_Status usb_low_SetPixelDeepth (ma1017 * chip,
					   PIXELDEEPTH pdPixelDeepth);
static SANE_Status usb_low_InvertImage (ma1017 * chip, SANE_Bool isInvert);
/* A17 + A18 + A19 */
static SANE_Status usb_low_GetA17 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_GetA18 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_GetA19 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetRedRef (ma1017 * chip, SANE_Byte RedRef);
static SANE_Status usb_low_SetGreenRef (ma1017 * chip, SANE_Byte GreenRef);
static SANE_Status usb_low_SetBlueRef (ma1017 * chip, SANE_Byte BlueRef);
/* A20 + A21 + A22 */
static SANE_Status usb_low_GetA20 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_GetA21 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_GetA22 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetRedPD (ma1017 * chip, SANE_Byte RedPD);
static SANE_Status usb_low_SetGreenPD (ma1017 * chip, SANE_Byte GreenPD);
static SANE_Status usb_low_SetBluePD (ma1017 * chip, SANE_Byte BluePD);
/* A23 */
static SANE_Status usb_low_GetA23 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_TurnPeripheralPower (ma1017 * chip,
						SANE_Bool isOn);
static SANE_Status usb_low_TurnLampPower (ma1017 * chip, SANE_Bool isOn);
static SANE_Status usb_low_SetIO3 (ma1017 * chip, SANE_Bool isHigh);
static SANE_Status usb_low_SetLEDLightAll (ma1017 * chip,
					   SANE_Bool isLightAll);
/* A24 */
static SANE_Status usb_low_GetA24 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetADTiming (ma1017 * chip, SANE_Byte bPattern);
/* A25 + A26 */
static SANE_Status usb_low_SetSerialByte1 (ma1017 * chip, SANE_Byte Byte1);
static SANE_Status usb_low_SetSerialByte2 (ma1017 * chip, SANE_Byte Byte2);
/* A27 */
static SANE_Status usb_low_GetA27 (ma1017 * chip, SANE_Byte *value);
static SANE_Status usb_low_SetSerialFormat (ma1017 * chip, SANE_Byte Format);
/* A31 */
static SANE_Status usb_low_GetHomeSensor (ma1017 * chip);
/* Special Mode */
static SANE_Status usb_low_StartRowing (ma1017 * chip);
static SANE_Status usb_low_StopRowing (ma1017 * chip);
static SANE_Status usb_low_WaitRowingStop (ma1017 * chip);
static SANE_Status usb_low_read_all_registers (ma1017 * chip);
static SANE_Status usb_low_SetCIS (ma1017 * chip, SENSOR Sensor);
static SANE_Status usb_low_GetRow (ma1017 * chip, SANE_Byte * pData, 
				   SANE_Word *LeftLine);
static SANE_Status usb_low_GetRowDirect (ma1017 * chip, SANE_Byte * pData,
					 SANE_Word *LeftLine);
static SANE_Status usb_low_GetRowResample (ma1017 * chip, SANE_Byte * pData,
					   SANE_Word *LeftLine);
/* Direct access */
static SANE_Status usb_low_wait_rowing (ma1017 * chip);
static SANE_Status usb_low_read_rows (ma1017 * chip, SANE_Byte * pData,
				      SANE_Word dwByteCount);
static SANE_Status usb_low_write_reg (ma1017 * chip, CONTROLWORD Control);
static SANE_Status usb_low_read_reg (ma1017 * chip, SANE_Byte RegNo,
				     SANE_Byte *data);
static SANE_Status usb_low_identify_scanner (SANE_Int fd,
					     Mustek_Type *scanner_type);
static SANE_Status usb_low_Open (ma1017 * chip, const char * devname);
static SANE_Status usb_low_Close (ma1017 * chip);

#endif /* defined mustek_usb_low_h */
