/* sane - Scanner Access Now Easy.

   Copyright (C) 2000 Mustek.
   Maintained by Tom Wang <tom.wang@mustek.com.tw>

   Updates (C) 2001 by Henning Meier-Geinitz.

   This file is part of the SANE package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.

   This file implements a SANE backend for Mustek 1200UB flatbed scanners.  */

#include "mustek_usb_mid.h"
#include "mustek_usb_low.c"


/* ---------------------- sensor 600 CIS functions ----------------------- */

static SANE_Word usb_mid_c600_s_wOpticalXDpi[] =
{ 600, 400, 300, 200, 150, 100, 50, 0 };

SANE_Status
usb_mid_c600_PrepareRgbX600Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareRgbX600Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P6P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareRgbX600Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareRgbX400Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareRgbX400Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P4P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareRgbX400Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareRgbX300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareRgbX300Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareRgbX300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareRgbX200Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareRgbX200Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P2P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareRgbX200Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareRgbX150Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareRgbX150Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6));
  RIE(usb_low_SetSoftResample (chip, 2));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareRgbX150Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareRgbX100Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareRgbX100Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareRgbX100Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareRgbX50Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareRgbX50Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6));
  RIE(usb_low_SetSoftResample (chip, 2));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareRgbX50Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareMonoX600Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareMonoX600Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P6P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareMonoX600Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareMonoX400Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareMonoX400Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P4P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareMonoX400Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareMonoX300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareMonoX300Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareMonoX300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareMonoX200Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareMonoX200Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P2P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareMonoX200Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareMonoX150Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareMonoX150Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6));
  RIE(usb_low_SetSoftResample (chip, 2));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareMonoX150Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareMonoX100Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareMonoX100Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareMonoX100Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareMonoX50Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_c600_PrepareMonoX50Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6));
  RIE(usb_low_SetSoftResample (chip, 2));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG (6, "usb_mid_c600_PrepareMonoX50Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareRgbX (ma1017 * chip, SANE_Word Dpi)
{
  DBG (6, "usb_mid_c600_PrepareRgbX: start\n");
  switch (Dpi)
    {
    case 50:
      return usb_mid_c600_PrepareRgbX50Dpi (chip);
      break;
    case 100:
      return usb_mid_c600_PrepareRgbX100Dpi (chip);
      break;
    case 150:
      return usb_mid_c600_PrepareRgbX150Dpi (chip);
      break;
    case 200:
      return usb_mid_c600_PrepareRgbX200Dpi (chip);
      break;
    case 300:
      return usb_mid_c600_PrepareRgbX300Dpi (chip);
      break;
    case 400:
      return usb_mid_c600_PrepareRgbX400Dpi (chip);
      break;
    case 600:
      return usb_mid_c600_PrepareRgbX600Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_c600_PrepareRgbX: unmatched DPI\n");
      return SANE_STATUS_INVAL;
      break;
    }
  DBG (6, "usb_mid_c600_PrepareRgbX: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c600_PrepareMonoX (ma1017 * chip, SANE_Word Dpi)
{
  DBG (6, "usb_mid_c600_PrepareMonoX: start\n");
  switch (Dpi)
    {
    case 50:
      return usb_mid_c600_PrepareMonoX50Dpi (chip);
      break;
    case 100:
      return usb_mid_c600_PrepareMonoX100Dpi (chip);
      break;
    case 150:
      return usb_mid_c600_PrepareMonoX150Dpi (chip);
      break;
    case 200:
      return usb_mid_c600_PrepareMonoX200Dpi (chip);
      break;
    case 300:
      return usb_mid_c600_PrepareMonoX300Dpi (chip);
      break;
    case 400:
      return usb_mid_c600_PrepareMonoX400Dpi (chip);
      break;
    case 600:
      return usb_mid_c600_PrepareMonoX600Dpi (chip);
      break;
    default:
      DBG (6, "usb_mid_c600_PrepareMonoX: unmatched DPI\n");
      return SANE_STATUS_INVAL;
      break;
    }
  return SANE_STATUS_GOOD;
}

SANE_Bool
usb_mid_c600_is600Mode (ma1017 * chip, SANE_Word Dpi)
{
  DBG (6, "usb_mid_c600_is600Mode: start, chip=%p, Dpi=%d, TRUE\n", chip, Dpi);
  /* Cannon 600 CIS is 600 dpi mode always. */
  return SANE_TRUE;
}


/* ------------------- sensor 300/600 CIS functions ----------------------- */


static SANE_Word usb_mid_c300600_s_wOpticalXDpi[] =
  { 600, 400, 300, 200, 150, 100, 50, 0 };

SANE_Status
usb_mid_c300600_PrepareRgbX600Dpi (ma1017 * chip)
{
  SANE_Status status;

  DBG(6, "usb_mid_c300600_PrepareRgbX600Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P6P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareRgbX600Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareRgbX400Dpi (ma1017 * chip)
{
  SANE_Status status;

  DBG(6, "usb_mid_c300600_PrepareRgbX400Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P4P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareRgbX400Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareRgbX300Dpi (ma1017 * chip)
{
  SANE_Status status;

  DBG(6, "usb_mid_c300600_PrepareRgbX300Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P6P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareRgbX300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareRgbX200Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300600_PrepareRgbX200Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P2P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareRgbX200Dpi: exit\n");
  return SANE_STATUS_GOOD;
}


SANE_Status
usb_mid_c300600_PrepareRgbX150Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300600_PrepareRgbX150Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareRgbX150Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareRgbX100Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300600_PrepareRgbX100Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P2P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareRgbX100Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareRgbX50Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300600_PrepareRgbX50Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareRgbX50Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareMonoX600Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300600_PrepareMonoX600Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P6P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareMonoX600Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareMonoX400Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300600_PrepareMonoX400Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P4P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareMonoX400Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareMonoX300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300600_PrepareMonoX300Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P6P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareMonoX300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareMonoX200Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300600_PrepareMonoX200Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P2P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareMonoX200Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareMonoX150Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300600_PrepareMonoX150Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareMonoX150Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareMonoX100Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300600_PrepareMonoX100Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P2P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareMonoX100Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareMonoX50Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300600_PrepareMonoX50Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300600_PrepareMonoX50Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareRgbX (ma1017 * chip, SANE_Word Dpi)
{
  DBG (6, "usb_mid_c300600_PrepareRgbX: start\n");
  switch (Dpi)
    {
    case 50:
      return usb_mid_c300600_PrepareRgbX50Dpi (chip);
      break;
    case 100:
      return usb_mid_c300600_PrepareRgbX100Dpi (chip);
      break;
    case 150:
      return usb_mid_c300600_PrepareRgbX150Dpi (chip);
      break;
    case 200:
      return usb_mid_c300600_PrepareRgbX200Dpi (chip);
      break;
    case 300:
      return usb_mid_c300600_PrepareRgbX300Dpi (chip);
      break;
    case 400:
      return usb_mid_c300600_PrepareRgbX400Dpi (chip);
      break;
    case 600:
      return usb_mid_c300600_PrepareRgbX600Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_c300600_PrepareRgbX: unmatched DPI?\n");
      return SANE_STATUS_INVAL;
      break;
    }
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300600_PrepareMonoX (ma1017 * chip, SANE_Word Dpi)
{
  switch (Dpi)
    {
    case 50:
      return usb_mid_c300600_PrepareMonoX50Dpi (chip);
      break;
    case 100:
      return usb_mid_c300600_PrepareMonoX100Dpi (chip);
      break;
    case 150:
      return usb_mid_c300600_PrepareMonoX150Dpi (chip);
      break;
    case 200:
      return usb_mid_c300600_PrepareMonoX200Dpi (chip);
      break;
    case 300:
      return usb_mid_c300600_PrepareMonoX300Dpi (chip);
      break;
    case 400:
      return usb_mid_c300600_PrepareMonoX400Dpi (chip);
      break;
    case 600:
      return usb_mid_c300600_PrepareMonoX600Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_c300600_PrepareMonoX: unmatched DPI?\n");
      return SANE_STATUS_INVAL;
      break;
    }
  return SANE_STATUS_GOOD;
}

SANE_Bool
usb_mid_c300600_is600Mode (ma1017 * chip, SANE_Word Dpi)
{
  switch (Dpi)
    {
    case 300:
    case 150:
    case 100:
    case 50:
      DBG (6, "usb_mid_c300600_is600Mode: chip=%p, Dpi=%d, FALSE\n", 
	   chip, Dpi);
      return SANE_FALSE;
    case 600:
    case 400:
    case 200:
      DBG (6, "usb_mid_c300600_is600Mode: chip=%p, Dpi=%d, TRUE\n", chip, Dpi);
      return SANE_TRUE;
    default:
      DBG (3, "usb_mid_c300600_is600Mode: unmatched DPI?\n");
      return SANE_FALSE;
      break;
    }
}

/* ---------------------- sensor 600 CIS functions ----------------------- */

static SANE_Word usb_mid_c300_s_wOpticalXDpi[] =
  { 600, 400, 300, 200, 150, 100, 50, 0 };

SANE_Status
usb_mid_c300_PrepareRgbX600Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareRgbX600Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P6P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareRgbX600Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareRgbX400Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareRgbX400Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P4P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareRgbX400Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareRgbX300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareRgbX300Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P6P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareRgbX300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareRgbX200Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareRgbX200Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P4P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareRgbX200Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareRgbX150Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareRgbX150Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareRgbX150Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareRgbX100Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareRgbX100Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P2P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareRgbX100Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareRgbX50Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareRgbX50Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareRgbX50Dpi: start\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareMonoX600Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareMonoX600Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P6P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareMonoX600Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareMonoX400Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareMonoX400Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P4P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareMonoX400Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareMonoX300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareMonoX300Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P6P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareMonoX300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareMonoX200Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareMonoX200Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_TRUE, SW_P4P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareMonoX200Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareMonoX150Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareMonoX150Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P3P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareMonoX150Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareMonoX100Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareMonoX100Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P2P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareMonoX100Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareMonoX50Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG(6, "usb_mid_c300_PrepareMonoX50Dpi: start\n");
  RIE(usb_low_SetImageDpi (chip, SANE_FALSE, SW_P1P6));
  RIE(usb_low_SetSoftResample (chip, 1));
  RIE(usb_low_SetLEDLightAll (chip, SANE_FALSE));
  DBG(6, "usb_mid_c300_PrepareMonoX50Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareRgbX (ma1017 * chip, SANE_Word Dpi)
{
  DBG (6, "usb_mid_c300_PrepareRgbX: start\n");
  switch (Dpi)
    {
    case 50:
      return usb_mid_c300_PrepareRgbX50Dpi (chip);
      break;
    case 100:
      return usb_mid_c300_PrepareRgbX100Dpi (chip);
      break;
    case 150:
      return usb_mid_c300_PrepareRgbX150Dpi (chip);
      break;
    case 200:
      return usb_mid_c300_PrepareRgbX200Dpi (chip);
      break;
    case 300:
      return usb_mid_c300_PrepareRgbX300Dpi (chip);
      break;
    case 400:
      return usb_mid_c300_PrepareRgbX400Dpi (chip);
      break;
    case 600:
      return usb_mid_c300_PrepareRgbX600Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_c300_PrepareRgbX: unmatched DPI?\n");
      return SANE_STATUS_INVAL;
      break;
    }
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_c300_PrepareMonoX (ma1017 * chip, SANE_Word Dpi)
{
  DBG (6, "usb_mid_c300_PrepareMonoX: start\n");
  switch (Dpi)
    {
    case 50:
      return usb_mid_c300_PrepareMonoX50Dpi (chip);
      break;
    case 100:
      return usb_mid_c300_PrepareMonoX100Dpi (chip);
      break;
    case 150:
      return usb_mid_c300_PrepareMonoX150Dpi (chip);
      break;
    case 200:
      return usb_mid_c300_PrepareMonoX200Dpi (chip);
      break;
    case 300:
      return usb_mid_c300_PrepareMonoX300Dpi (chip);
      break;
    case 400:
      return usb_mid_c300_PrepareMonoX400Dpi (chip);
      break;
    case 600:
      return usb_mid_c300_PrepareMonoX600Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_c300_PrepareMonoX: unmatched DPI?\n");
      return SANE_STATUS_INVAL;
      break;
    }
  return SANE_STATUS_GOOD;
}

SANE_Bool
usb_mid_c300_is600Mode (ma1017 * chip, SANE_Word Dpi)
{
  DBG (6, "usb_mid_c300_is600Mode: chip=%p, Dpi=%d, FALSE\n", chip, Dpi);
  return SANE_FALSE;
}


/* ---------------1200 dpi motor function declarations --------------------- */

static SANE_Word usb_mid_motor1200_s_wOpticalYDpi[] =
{ 1200, 600, 400, 300, 240, 200, 150, 120, 100, 75, 60, 50, 0 };

SANE_Status
usb_mid_motor1200_PrepareRgbY1200Dpi (ma1017 * chip)
{
  SANE_Status status;

  DBG (6, "usb_mid_motor1200_PrepareRgbY1200Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 4));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_FALSE));
  DBG (6, "usb_mid_motor1200_PrepareRgbY1200Dpi: exit\n");
  return SANE_STATUS_GOOD;
}


SANE_Status
usb_mid_motor1200_PrepareRgbY600Dpi (ma1017 * chip)
{
  SANE_Status status;

  DBG (6, "usb_mid_motor1200_PrepareRgbY600Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 4));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_FALSE));
  DBG (6, "usb_mid_motor1200_PrepareRgbY600Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareRgbY400Dpi (ma1017 * chip)
{
  SANE_Status status;

  DBG (6, "usb_mid_motor1200_PrepareRgbY400Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 3));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareRgbY400Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareRgbY300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareRgbY300Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 4));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareRgbY300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareRgbY200Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareRgbY200Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 3));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareRgbY200Dpi: exit\n");
  return SANE_STATUS_GOOD;
}



SANE_Status
usb_mid_motor1200_PrepareRgbY150Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareRgbY150Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 4));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareRgbY150Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareRgbY100Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareRgbY100Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_BLUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_RED, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 3));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareRgbY100Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareRgbY50Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareRgbY50Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_BLUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_BLUE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_RED, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 5, CH_RED, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 6));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareRgbY50Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareMonoY1200Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareMonoY1200Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 2));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareMonoY1200Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareMonoY600Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareMonoY600Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 7, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 8, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 9, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 10, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 11, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 12, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 13, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 14, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 15, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 16, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 17, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 18, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 19, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 20, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 21, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 22, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 23, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 24, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 25, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 26, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 26));
  RIE(usb_low_SetCMTSecondPosition (chip, 24));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareMonoY600Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareMonoY400Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareMonoY400Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 3));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareMonoY400Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareMonoY300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareMonoY300Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 7, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 8, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 9, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 10, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 11, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 12, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 13, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 14, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 15, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 16, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 17, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 18, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 19, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 20, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 21, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 22, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 23, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 24, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 25, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 26, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 26));
  RIE(usb_low_SetCMTSecondPosition (chip, 24));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareMonoY300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareMonoY200Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareMonoY200Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 3));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareMonoY200Dpi: exit\n");
  return SANE_STATUS_GOOD;
}


SANE_Status
usb_mid_motor1200_PrepareMonoY150Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareMonoY150Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 2));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareMonoY150Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareMonoY100Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareMonoY100Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 3));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareMonoY100Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareMonoY50Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareMonoY50Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 6));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareMonoY50Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareRgbYHalf300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareRgbYHalf300Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_BLUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_BLUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_RED, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 5, CH_RED, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 6));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareRgbYHalf300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareRgbYBiFull300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareRgbYBiFull300Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_BLUE, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_BLUE, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_RED, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 5, CH_RED, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 6));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_FALSE));
  DBG (6, "usb_mid_motor1200_PrepareRgbYBiFull300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareRgbYBiFullx2300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareRgbYBiFullx2300Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_BLUE, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_BLUE, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_RED, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 5, CH_RED, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 6));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareRgbYBiFullx2300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareMonoYHalf300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareMonoYHalf300Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 4));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareMonoYHalf300Dpi: exit\n");
  return SANE_STATUS_GOOD;

}

SANE_Status
usb_mid_motor1200_PrepareMonoYBiFull300Dpi (ma1017 * chip)
{
  SANE_Status status;

  DBG (6, "usb_mid_motor1200_PrepareMonoYBiFull300Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 2));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareMonoYBiFull300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareMonoYBiFullx2300Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareMonoYBiFullx2300Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 2));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareMonoYBiFullx2300Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareRgbY (ma1017 * chip, SANE_Word Dpi)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareMonoRgbY: start\n");
  RIE(usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE));
  /* No Motor & Forward */
  RIE(usb_low_SetMotorDirection (chip, SANE_FALSE));
  RIE(usb_low_EnableMotor (chip, SANE_TRUE));
  switch (Dpi)
    {
    case 1200:
      return usb_mid_motor1200_PrepareRgbY1200Dpi (chip);
      break;
    case 600:
      return usb_mid_motor1200_PrepareRgbY600Dpi (chip);
      break;
    case 400:
      return usb_mid_motor1200_PrepareRgbY400Dpi (chip);
      break;
    case 300:
      return usb_mid_motor1200_PrepareRgbY300Dpi (chip);
      break;
    case 200:
      return usb_mid_motor1200_PrepareRgbY200Dpi (chip);
      break;
    case 150:
      return usb_mid_motor1200_PrepareRgbY150Dpi (chip);
      break;
    case 100:
      return usb_mid_motor1200_PrepareRgbY100Dpi (chip);
      break;
    case 50:
      return usb_mid_motor1200_PrepareRgbY50Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_motor1200_PrepareRgbY: unmatched Dpi?\n");
      return SANE_STATUS_INVAL;
      break;
    }
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareMonoY (ma1017 * chip, SANE_Word Dpi)
{
  SANE_Status status;
  
  DBG (3, "usb_mid_motor1200_PrepareMonoY: start\n");
  RIE(usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE));
  /* No Motor & Forward */
  RIE(usb_low_SetMotorDirection (chip, SANE_FALSE));
  RIE(usb_low_EnableMotor (chip, SANE_TRUE));
  switch (Dpi)
    {
    case 1200:
      return usb_mid_motor1200_PrepareMonoY1200Dpi (chip);
      break;
    case 600:
      return usb_mid_motor1200_PrepareMonoY600Dpi (chip);
      break;
    case 400:
      return usb_mid_motor1200_PrepareMonoY400Dpi (chip);
      break;
    case 300:
      return usb_mid_motor1200_PrepareMonoY300Dpi (chip);
      break;
    case 200:
      return usb_mid_motor1200_PrepareMonoY200Dpi (chip);
      break;
    case 150:
      return usb_mid_motor1200_PrepareMonoY150Dpi (chip);
      break;
    case 100:
      return usb_mid_motor1200_PrepareMonoY100Dpi (chip);
      break;
    case 50:
      return usb_mid_motor1200_PrepareMonoY50Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_motor1200_PrepareMonoY: unmatched Dpi?\n");
      return SANE_STATUS_INVAL;
      break;
    }
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareCalibrateRgbY (ma1017 * chip, SANE_Word Dpi)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareCalibrateRgbY: start\n");
  RIE(usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE));
  /* No Motor & Forward */
  RIE(usb_low_SetMotorDirection (chip, SANE_FALSE));
  RIE(usb_low_EnableMotor (chip, SANE_TRUE));
  switch (Dpi)
    {
    case 1200:
    case 400:
    case 300:
      return usb_mid_motor1200_PrepareRgbYHalf300Dpi (chip);
      break;
    case 600:
    case 200:
    case 150:
      return usb_mid_motor1200_PrepareRgbYBiFull300Dpi (chip);
      break;
    case 100:
    case 50:
      return usb_mid_motor1200_PrepareRgbYBiFullx2300Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_motor1200_PrepareCalibrateRgbY: unmatched Dpi?\n");
      return SANE_STATUS_INVAL;
      break;
    }
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareCalibrateMonoY (ma1017 * chip, SANE_Word Dpi)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareCalibrateMonoY: start\n");
  RIE(usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE));
  /* No Motor & Forward */
  RIE(usb_low_SetMotorDirection (chip, SANE_FALSE));
  RIE(usb_low_EnableMotor (chip, SANE_TRUE));
  switch (Dpi)
    {
    case 1200:
    case 600:
    case 400:
      return usb_mid_motor1200_PrepareMonoYHalf300Dpi (chip);
      break;
    case 300:
    case 200:
      return usb_mid_motor1200_PrepareMonoYBiFull300Dpi (chip);
      break;
    case 150:
    case 100:
    case 50:
      return usb_mid_motor1200_PrepareMonoYBiFullx2300Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_motor1200_PrepareCalibrateMonoY: unmatched Dpi?\n");
      return SANE_STATUS_INVAL;
      break;
    }
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareStep (ma1017 * chip, SANE_Word StepCount)
{
  SANE_Status status;

  DBG (6, "usb_mid_motor1200_PrepareStep: start\n");
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE));
  /* Make it in 600dpi */
  RIE(usb_low_SetIO3 (chip, SANE_TRUE)); /* (IO3) ? High power : Low power */
  RIE(usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE));
  /* No Motor & Forward */
  if (StepCount == 1)
    {
      RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_FALSE));
      RIE(usb_low_SetCMTTableLength (chip, 1));
      RIE(usb_low_SetCMTSecondPosition (chip, 0));
      RIE(usb_low_SetCMTLoopCount (chip, StepCount));
    }
  else if (StepCount % 2 == 1)
    {
      RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
      RIE(usb_low_SetCMTTableLength (chip, 3));
      RIE(usb_low_SetCMTSecondPosition (chip, 1));
      RIE(usb_low_SetCMTLoopCount (chip, (StepCount - 1) / 2));
    }
  else
    {
      RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE));
      RIE(usb_low_SetCMTTableLength (chip, 2));
      RIE(usb_low_SetCMTSecondPosition (chip, 0));
      RIE(usb_low_SetCMTLoopCount (chip, StepCount / 2));
    }
  RIE(usb_low_EnableMotor (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareStep: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareHome (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareHome: start\n");
  RIE(usb_low_SetMotorMovement (chip, SANE_TRUE, SANE_TRUE, SANE_FALSE));
  /* Make it in 600dpi */
  RIE(usb_low_SetIO3 (chip, SANE_TRUE)); /* (IO3) ? High power : Low power */
  RIE(usb_low_MoveMotorHome (chip, SANE_TRUE, SANE_TRUE));
  DBG (6, "usb_mid_motor1200_PrepareHome: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor1200_PrepareAdjust (ma1017 * chip, CHANNEL chSelect)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor1200_PrepareAdjust: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, chSelect, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, chSelect, SANE_FALSE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 2, chSelect, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 2));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  DBG (6, "usb_mid_motor1200_PrepareAdjust: exit\n");
  return SANE_STATUS_GOOD;
}


SANE_Word
usb_mid_motor1200_RgbCapability (SANE_Word Dpi)
{
  DBG (6, "usb_mid_motor1200_RgbCapability: start\n");
  switch (Dpi)
    {
    case 1200:
    case 400:
      return 3008;		/* 2816 */; 
    case 600:
      return 3008;
    case 200:
      return 5056;
    case 300:
      return 3008;
    case 150:
      return 5056;
    case 100:
    case 50:
      return 10048;
    default:
      DBG (3, "usb_mid_motor1200_RgbCapability: unmatched DPI?\n");
      return 0;
    }
}

SANE_Word
usb_mid_motor1200_MonoCapability (SANE_Word Dpi)
{
  DBG (5, "usb_mid_motor1200_MonoCapability: start\n");
  switch (Dpi)
    {
    case 1200:
    case 400:
      return 3008;
    case 600:
      return 3008;
    case 200:
      return 5056;
    case 300:
      return 5056;
    case 150:
    case 100:
    case 50:
      return 10048;
    default:
      DBG (3, "usb_mid_motor1200_MonoCapability: unmatched Dpi?\n");
      return 0;
    }
}

/* ---------------600 dpi motor function declarations --------------------- */

SANE_Status
usb_mid_motor600_PrepareMonoY200Dpi (ma1017 * chip)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor600_PrepareMonoY200Dpi: start\n");
  RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_TRUE));
  RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 4, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 5, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 6, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 7, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 8, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 9, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 10, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 11, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 12, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 13, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 14, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 15, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 16, CH_GREEN, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 17, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 18, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 19, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 20, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 21, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 22, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 23, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTable (chip, 24, CH_GREEN, SANE_FALSE, SANE_FALSE));
  RIE(usb_low_SetCMTTableLength (chip, 24));
  RIE(usb_low_SetCMTSecondPosition (chip, 0));
  RIE(usb_low_SetCMTLoopCount (chip, 0xefff));
  RIE(usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE));
  RIE(usb_low_SetIO3 (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor600_PrepareMonoY200Dpi: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor600_PrepareRgbY (ma1017 * chip, SANE_Word Dpi)
{
  SANE_Status status;
  
  DBG (6, "usb_mid_motor600_PrepareMonoRgbY: start\n");
  RIE(usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE));
  /* No Motor & Forward */
  RIE(usb_low_SetMotorDirection (chip, SANE_FALSE));
  RIE(usb_low_EnableMotor (chip, SANE_TRUE));
  switch (Dpi)
    {
    case 300:
      return usb_mid_motor1200_PrepareRgbY600Dpi (chip);
      break;
    case 200:
      return usb_mid_motor1200_PrepareRgbY400Dpi (chip);
      break;
    case 150:
      return usb_mid_motor1200_PrepareRgbY300Dpi (chip);
      break;
    case 100:
      return usb_mid_motor1200_PrepareRgbY200Dpi (chip);
      break;
    case 50:
      return usb_mid_motor1200_PrepareRgbY100Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_motor600_PrepareRgbY: unmatched Dpi (%d)\n", Dpi);
      return SANE_STATUS_INVAL;
      break;
    }
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor600_PrepareMonoY (ma1017 * chip, SANE_Word Dpi)
{
  SANE_Status status;
  
  DBG (3, "usb_mid_motor600_PrepareMonoY: start\n");
  RIE(usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE));
  /* No Motor & Forward */
  RIE(usb_low_SetMotorDirection (chip, SANE_FALSE));
  RIE(usb_low_EnableMotor (chip, SANE_TRUE));
  switch (Dpi)
    {
    case 300:
      return usb_mid_motor1200_PrepareMonoY600Dpi (chip);
      break;
    case 200:
      return usb_mid_motor600_PrepareMonoY200Dpi (chip);
      break;
    case 150:
      return usb_mid_motor1200_PrepareMonoY300Dpi (chip);
      break;
    case 100:
      return usb_mid_motor1200_PrepareMonoY200Dpi (chip);
      break;
    case 50:
      return usb_mid_motor1200_PrepareMonoY100Dpi (chip);
      break;
    default:
      DBG (3, "usb_mid_motor600_PrepareMonoY: unmatched Dpi (%d)\n", Dpi);
      return SANE_STATUS_INVAL;
      break;
    }
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_motor600_PrepareStep (ma1017 * chip, SANE_Word StepCount)
{
  SANE_Status status;

  DBG (6, "usb_mid_motor600_PrepareStep: start\n");
  RIE(usb_low_SetMotorMovement (chip, SANE_FALSE, SANE_TRUE, SANE_FALSE));
  /* Make it in 600dpi */
  RIE(usb_low_SetIO3 (chip, SANE_TRUE)); /* (IO3) ? High power : Low power */
  RIE(usb_low_MoveMotorHome (chip, SANE_FALSE, SANE_FALSE));
  /* No Motor & Forward */
  if (StepCount == 1)
    {
      RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_FALSE, SANE_FALSE));
      RIE(usb_low_SetCMTTableLength (chip, 1));
      RIE(usb_low_SetCMTSecondPosition (chip, 0));
      RIE(usb_low_SetCMTLoopCount (chip, StepCount));
    }
  else if (StepCount % 2 == 1)
    {
      RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 3, CH_GREEN, SANE_FALSE, SANE_FALSE));
      RIE(usb_low_SetCMTTableLength (chip, 3));
      RIE(usb_low_SetCMTSecondPosition (chip, 1));
      RIE(usb_low_SetCMTLoopCount (chip, (StepCount - 1) / 2));
    }
  else
    {
      RIE(usb_low_SetCMTTable (chip, 0, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 1, CH_GREEN, SANE_TRUE, SANE_FALSE));
      RIE(usb_low_SetCMTTable (chip, 2, CH_GREEN, SANE_FALSE, SANE_FALSE));
      RIE(usb_low_SetCMTTableLength (chip, 2));
      RIE(usb_low_SetCMTSecondPosition (chip, 0));
      RIE(usb_low_SetCMTLoopCount (chip, StepCount / 2));
    }
  RIE(usb_low_EnableMotor (chip, SANE_TRUE));
  DBG (6, "usb_mid_motor600_PrepareStep: exit\n");
  return SANE_STATUS_GOOD;
}
/* ----------------------------- frontend ------------------------------- */

SANE_Status
usb_mid_front_SetFrontEndMode (ma1017 * chip, SANE_Byte Mode)
{
  SANE_Status status;

  DBG (6, "usb_mid_front_SetFrontEndMode: start\n");
  RIE(usb_low_SetSerialFormat (chip, Mode));
  DBG (6, "usb_mid_front_SetFrontEndMode: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_front_Enable (ma1017 * chip, SANE_Bool isEnable)
{
  SANE_Status status;

  DBG (6, "usb_mid_front_Enable: start\n");
  RIE(usb_low_TurnFrontEndMode (chip, isEnable));
  DBG (6, "usb_mid_front_Enable: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_front_SetTopReference (ma1017 * chip, SANE_Byte Top)
{
  SANE_Status status;

  DBG (6, "usb_mid_front_SetTopReference: start\n");
  RIE(usb_mid_front_Enable (chip, SANE_TRUE));
  RIE(usb_low_SetSerialByte1 (chip, 0x00));
  RIE(usb_low_SetSerialByte2 (chip, Top));
  RIE(usb_mid_front_Enable (chip, SANE_FALSE));
  DBG (6, "usb_mid_front_SetTopReference: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_front_SetRedOffset (ma1017 * chip, SANE_Byte Offset)
{
  SANE_Status status;

  DBG (6, "usb_mid_front_SetRedOffset: start\n");
  RIE(usb_mid_front_Enable (chip, SANE_TRUE));
  RIE(usb_low_SetSerialByte1 (chip, 0x10));
  RIE(usb_low_SetSerialByte2 (chip, Offset));
  RIE(usb_mid_front_Enable (chip, SANE_FALSE));
  DBG (6, "usb_mid_front_SetRedOffset: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_front_SetGreenOffset (ma1017 * chip, SANE_Byte Offset)
{
  SANE_Status status;

  DBG (6, "usb_mid_front_SetGreenOffset: start\n");
  RIE(usb_mid_front_Enable (chip, SANE_TRUE));
  RIE(usb_low_SetSerialByte1 (chip, 0x50));
  RIE(usb_low_SetSerialByte2 (chip, Offset));
  RIE(usb_mid_front_Enable (chip, SANE_FALSE));
  DBG (6, "usb_mid_front_SetGreenOffset: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_front_SetBlueOffset (ma1017 * chip, SANE_Byte Offset)
{
  SANE_Status status;

  DBG (6, "usb_mid_front_SetBlueOffset: start\n");
  RIE(usb_mid_front_Enable (chip, SANE_TRUE));
  RIE(usb_low_SetSerialByte1 (chip, 0x30));
  RIE(usb_low_SetSerialByte2 (chip, Offset));
  RIE(usb_mid_front_Enable (chip, SANE_FALSE));
  DBG (6, "usb_mid_front_SetBlueOffset: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_front_SetRedPGA (ma1017 * chip, SANE_Byte PGA)
{
  SANE_Status status;

  DBG (6, "usb_mid_front_SetRedPGA: start\n");
  RIE(usb_mid_front_Enable (chip, SANE_TRUE));
  RIE(usb_low_SetSerialByte1 (chip, 0x40));
  RIE(usb_low_SetSerialByte2 (chip, PGA));
  RIE(usb_mid_front_Enable (chip, SANE_FALSE));
  DBG (6, "usb_mid_front_SetRedPGA: start\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_front_SetGreenPGA (ma1017 * chip, SANE_Byte PGA)
{
  SANE_Status status;

  DBG (6, "usb_mid_front_SetGreenPGA: start\n");
  RIE(usb_mid_front_Enable (chip, SANE_TRUE));
  RIE(usb_low_SetSerialByte1 (chip, 0x20));
  RIE(usb_low_SetSerialByte2 (chip, PGA));
  RIE(usb_mid_front_Enable (chip, SANE_FALSE));
  DBG (6, "usb_mid_front_SetGreenPGA: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_front_SetBluePGA (ma1017 * chip, SANE_Byte PGA)
{
  SANE_Status status;

  DBG (6, "usb_mid_front_SetBluePGA: start\n");
  RIE(usb_mid_front_Enable (chip, SANE_TRUE));
  RIE(usb_low_SetSerialByte1 (chip, 0x60));
  RIE(usb_low_SetSerialByte2 (chip, PGA));
  RIE(usb_mid_front_Enable (chip, SANE_FALSE));
  DBG (6, "usb_mid_front_SetBluePGA: exit\n");
  return SANE_STATUS_GOOD;
}

SANE_Status
usb_mid_front_SetRGBSignal (ma1017 * chip)
{
  SANE_Status status;

  DBG (6, "usb_mid_front_SetRGBSignal: start\n");
  RIE(usb_low_SetRedRef (chip, 0xEF));
  RIE(usb_low_SetGreenRef (chip, 0xF7));
  RIE(usb_low_SetBlueRef (chip, 0xFF));
  DBG (6, "usb_mid_front_SetRGBSignal: exit\n");
  return SANE_STATUS_GOOD;
}

